"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Reporting of performance timeline events, as specified in
https://w3c.github.io/performance-timeline/#dom-performanceobserver.
https://chromedevtools.github.io/devtools-protocol/tot/PerformanceTimeline/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol.base import CDPMethod, InputModel


class EnableInput(InputModel):

    eventTypes: list[str]


class Enable(CDPMethod[None]):
    """ Previously buffered events would be reported before method returns.
    See also: timelineEventAdded """

    INPUT_VALIDATOR = EnableInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        event_types: list[str]
    ):
        super().__init__(
            eventTypes=event_types
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain is deprecated.
https://chromedevtools.github.io/devtools-protocol/tot/Schema/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Schema,
)
from cdpkit.protocol.base import CDPMethod, OutputModel


class GetDomainsOutput(OutputModel):

    domains: list[Schema.Domain]


class GetDomains(CDPMethod[GetDomainsOutput]):
    """ Returns supported domains. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetDomainsOutput



from .events import (
    AttachedToTarget,
    DetachedFromTarget,
    ReceivedMessageFromTarget,
    TargetCrashed,
    TargetCreated,
    TargetDestroyed,
    TargetInfoChanged,
)
from .methods import (
    ActivateTarget,
    AttachToBrowserTarget,
    AttachToTarget,
    AutoAttachRelated,
    CloseTarget,
    CreateBrowserContext,
    CreateTarget,
    DetachFromTarget,
    DisposeBrowserContext,
    ExposeDevToolsProtocol,
    GetBrowserContexts,
    GetDevToolsTarget,
    GetTargetInfo,
    GetTargets,
    OpenDevTools,
    SendMessageToTarget,
    SetAutoAttach,
    SetDiscoverTargets,
    SetRemoteLocations,
)
from .types import FilterEntry, RemoteLocation, SessionID, TargetFilter, TargetID, TargetInfo, WindowState

__all__ = [
    TargetID,
    SessionID,
    TargetInfo,
    FilterEntry,
    TargetFilter,
    RemoteLocation,
    WindowState,
    AttachedToTarget,
    DetachedFromTarget,
    ReceivedMessageFromTarget,
    TargetCreated,
    TargetDestroyed,
    TargetCrashed,
    TargetInfoChanged,
    ActivateTarget,
    AttachToTarget,
    AttachToBrowserTarget,
    CloseTarget,
    ExposeDevToolsProtocol,
    CreateBrowserContext,
    GetBrowserContexts,
    CreateTarget,
    DetachFromTarget,
    DisposeBrowserContext,
    GetTargetInfo,
    GetTargets,
    SendMessageToTarget,
    SetAutoAttach,
    AutoAttachRelated,
    SetDiscoverTargets,
    SetRemoteLocations,
    GetDevToolsTarget,
    OpenDevTools,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain facilitates obtaining document snapshots with DOM, layout, and style information.
https://chromedevtools.github.io/devtools-protocol/tot/DOMSnapshot/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    DOMSnapshotArrayOfStrings,
    DOMSnapshotComputedStyle,
    DOMSnapshotDocumentSnapshot,
    DOMSnapshotDOMNode,
    DOMSnapshotInlineTextBox,
    DOMSnapshotLayoutTreeNode,
    DOMSnapshotLayoutTreeSnapshot,
    DOMSnapshotNameValue,
    DOMSnapshotNodeTreeSnapshot,
    DOMSnapshotRareBooleanData,
    DOMSnapshotRareIntegerData,
    DOMSnapshotRareStringData,
    DOMSnapshotRectangle,
    DOMSnapshotStringIndex,
    DOMSnapshotTextBoxSnapshot,
)

DOMNode = DOMSnapshotDOMNode
InlineTextBox = DOMSnapshotInlineTextBox
LayoutTreeNode = DOMSnapshotLayoutTreeNode
ComputedStyle = DOMSnapshotComputedStyle
NameValue = DOMSnapshotNameValue
StringIndex = DOMSnapshotStringIndex
ArrayOfStrings = DOMSnapshotArrayOfStrings
RareStringData = DOMSnapshotRareStringData
RareBooleanData = DOMSnapshotRareBooleanData
RareIntegerData = DOMSnapshotRareIntegerData
Rectangle = DOMSnapshotRectangle
DocumentSnapshot = DOMSnapshotDocumentSnapshot
NodeTreeSnapshot = DOMSnapshotNodeTreeSnapshot
LayoutTreeSnapshot = DOMSnapshotLayoutTreeSnapshot
TextBoxSnapshot = DOMSnapshotTextBoxSnapshot



from .events import IssueUpdated, SinksUpdated
from .methods import Disable, Enable, SetSinkToUse, StartDesktopMirroring, StartTabMirroring, StopCasting
from .types import Sink

__all__ = [
    Sink,
    SinksUpdated,
    IssueUpdated,
    Enable,
    Disable,
    SetSinkToUse,
    StartDesktopMirroring,
    StartTabMirroring,
    StopCasting,
]



from .events import BufferUsage, DataCollected, TracingComplete
from .methods import End, GetCategories, GetTrackEventDescriptor, RecordClockSyncMarker, RequestMemoryDump, Start
from .types import (
    MemoryDumpConfig,
    MemoryDumpLevelOfDetail,
    StreamCompression,
    StreamFormat,
    TraceConfig,
    TracingBackend,
)

__all__ = [
    MemoryDumpConfig,
    TraceConfig,
    StreamFormat,
    StreamCompression,
    MemoryDumpLevelOfDetail,
    TracingBackend,
    BufferUsage,
    DataCollected,
    TracingComplete,
    End,
    GetCategories,
    GetTrackEventDescriptor,
    RecordClockSyncMarker,
    RequestMemoryDump,
    Start,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain allows detailed inspection of media elements.
https://chromedevtools.github.io/devtools-protocol/tot/Media/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol.base import CDPMethod


class Enable(CDPMethod[None]):
    """ Enables the Media domain """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Disable(CDPMethod[None]):
    """ Disables the Media domain. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain provides various functionality related to drawing atop the inspected page.
https://chromedevtools.github.io/devtools-protocol/tot/Overlay/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    DOM,
    Overlay,
    Page,
    Runtime,
)
from cdpkit.protocol.base import JSON_DICT, CDPMethod, InputModel, OutputModel


class Disable(CDPMethod[None]):
    """ Disables domain notifications. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables domain notifications. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class GetHighlightObjectForTestInput(InputModel):

    nodeId: DOM.NodeId
    includeDistance: bool | None = None
    includeStyle: bool | None = None
    colorFormat: Overlay.ColorFormat | None = None
    showAccessibilityInfo: bool | None = None


class GetHighlightObjectForTestOutput(OutputModel):

    highlight: JSON_DICT


class GetHighlightObjectForTest(CDPMethod[GetHighlightObjectForTestOutput]):
    """ For testing. """

    INPUT_VALIDATOR = GetHighlightObjectForTestInput
    OUTPUT_VALIDATOR = GetHighlightObjectForTestOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        include_distance: bool | None = None,
        include_style: bool | None = None,
        color_format: Overlay.ColorFormat | None = None,
        show_accessibility_info: bool | None = None
    ):
        super().__init__(
            nodeId=node_id,
            includeDistance=include_distance,
            includeStyle=include_style,
            colorFormat=color_format,
            showAccessibilityInfo=show_accessibility_info
        )


class GetGridHighlightObjectsForTestInput(InputModel):

    nodeIds: list[DOM.NodeId]


class GetGridHighlightObjectsForTestOutput(OutputModel):

    highlights: JSON_DICT


class GetGridHighlightObjectsForTest(CDPMethod[GetGridHighlightObjectsForTestOutput]):
    """ For Persistent Grid testing. """

    INPUT_VALIDATOR = GetGridHighlightObjectsForTestInput
    OUTPUT_VALIDATOR = GetGridHighlightObjectsForTestOutput

    def __init__(
        self,
        *,
        node_ids: list[DOM.NodeId]
    ):
        super().__init__(
            nodeIds=node_ids
        )


class GetSourceOrderHighlightObjectForTestInput(InputModel):

    nodeId: DOM.NodeId


class GetSourceOrderHighlightObjectForTestOutput(OutputModel):

    highlight: JSON_DICT


class GetSourceOrderHighlightObjectForTest(CDPMethod[GetSourceOrderHighlightObjectForTestOutput]):
    """ For Source Order Viewer testing. """

    INPUT_VALIDATOR = GetSourceOrderHighlightObjectForTestInput
    OUTPUT_VALIDATOR = GetSourceOrderHighlightObjectForTestOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId
    ):
        super().__init__(
            nodeId=node_id
        )


class HideHighlight(CDPMethod[None]):
    """ Hides any highlight. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class HighlightFrameInput(InputModel):

    frameId: Page.FrameId
    contentColor: DOM.RGBA | None = None
    contentOutlineColor: DOM.RGBA | None = None


class HighlightFrame(CDPMethod[None]):  # deprecated
    """ Highlights owner element of the frame with given id.
    Deprecated: Doesn't work reliably and cannot be fixed due to process
    separation (the owner node might be in a different process). Determine
    the owner node in the client and use highlightNode. """

    INPUT_VALIDATOR = HighlightFrameInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        frame_id: Page.FrameId,
        content_color: DOM.RGBA | None = None,
        content_outline_color: DOM.RGBA | None = None
    ):
        super().__init__(
            frameId=frame_id,
            contentColor=content_color,
            contentOutlineColor=content_outline_color
        )


class HighlightNodeInput(InputModel):

    highlightConfig: Overlay.HighlightConfig
    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectId: Runtime.RemoteObjectId | None = None
    selector: str | None = None


class HighlightNode(CDPMethod[None]):
    """ Highlights DOM node with given id or with the given JavaScript object wrapper. Either nodeId or
    objectId must be specified. """

    INPUT_VALIDATOR = HighlightNodeInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        highlight_config: Overlay.HighlightConfig,
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_id: Runtime.RemoteObjectId | None = None,
        selector: str | None = None
    ):
        super().__init__(
            highlightConfig=highlight_config,
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectId=object_id,
            selector=selector
        )


class HighlightQuadInput(InputModel):

    quad: DOM.Quad
    color: DOM.RGBA | None = None
    outlineColor: DOM.RGBA | None = None


class HighlightQuad(CDPMethod[None]):
    """ Highlights given quad. Coordinates are absolute with respect to the main frame viewport. """

    INPUT_VALIDATOR = HighlightQuadInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        quad: DOM.Quad,
        color: DOM.RGBA | None = None,
        outline_color: DOM.RGBA | None = None
    ):
        super().__init__(
            quad=quad,
            color=color,
            outlineColor=outline_color
        )


class HighlightRectInput(InputModel):

    x: int
    y: int
    width: int
    height: int
    color: DOM.RGBA | None = None
    outlineColor: DOM.RGBA | None = None


class HighlightRect(CDPMethod[None]):
    """ Highlights given rectangle. Coordinates are absolute with respect to the main frame viewport.
    Issue: the method does not handle device pixel ratio (DPR) correctly.
    The coordinates currently have to be adjusted by the client
    if DPR is not 1 (see crbug.com/437807128). """

    INPUT_VALIDATOR = HighlightRectInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        x: int,
        y: int,
        width: int,
        height: int,
        color: DOM.RGBA | None = None,
        outline_color: DOM.RGBA | None = None
    ):
        super().__init__(
            x=x,
            y=y,
            width=width,
            height=height,
            color=color,
            outlineColor=outline_color
        )


class HighlightSourceOrderInput(InputModel):

    sourceOrderConfig: Overlay.SourceOrderConfig
    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectId: Runtime.RemoteObjectId | None = None


class HighlightSourceOrder(CDPMethod[None]):
    """ Highlights the source order of the children of the DOM node with given id or with the given
    JavaScript object wrapper. Either nodeId or objectId must be specified. """

    INPUT_VALIDATOR = HighlightSourceOrderInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        source_order_config: Overlay.SourceOrderConfig,
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_id: Runtime.RemoteObjectId | None = None
    ):
        super().__init__(
            sourceOrderConfig=source_order_config,
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectId=object_id
        )


class SetInspectModeInput(InputModel):

    mode: Overlay.InspectMode
    highlightConfig: Overlay.HighlightConfig | None = None


class SetInspectMode(CDPMethod[None]):
    """ Enters the 'inspect' mode. In this mode, elements that user is hovering over are highlighted.
    Backend then generates 'inspectNodeRequested' event upon element selection. """

    INPUT_VALIDATOR = SetInspectModeInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        mode: Overlay.InspectMode,
        highlight_config: Overlay.HighlightConfig | None = None
    ):
        super().__init__(
            mode=mode,
            highlightConfig=highlight_config
        )


class SetShowAdHighlightsInput(InputModel):

    show: bool


class SetShowAdHighlights(CDPMethod[None]):
    """ Highlights owner element of all frames detected to be ads. """

    INPUT_VALIDATOR = SetShowAdHighlightsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        show: bool
    ):
        super().__init__(
            show=show
        )


class SetPausedInDebuggerMessageInput(InputModel):

    message: str | None = None


class SetPausedInDebuggerMessage(CDPMethod[None]):

    INPUT_VALIDATOR = SetPausedInDebuggerMessageInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        message: str | None = None
    ):
        super().__init__(
            message=message
        )


class SetShowDebugBordersInput(InputModel):

    show: bool


class SetShowDebugBorders(CDPMethod[None]):
    """ Requests that backend shows debug borders on layers """

    INPUT_VALIDATOR = SetShowDebugBordersInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        show: bool
    ):
        super().__init__(
            show=show
        )


class SetShowFPSCounterInput(InputModel):

    show: bool


class SetShowFPSCounter(CDPMethod[None]):
    """ Requests that backend shows the FPS counter """

    INPUT_VALIDATOR = SetShowFPSCounterInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        show: bool
    ):
        super().__init__(
            show=show
        )


class SetShowGridOverlaysInput(InputModel):

    gridNodeHighlightConfigs: list[Overlay.GridNodeHighlightConfig]


class SetShowGridOverlays(CDPMethod[None]):
    """ Highlight multiple elements with the CSS Grid overlay. """

    INPUT_VALIDATOR = SetShowGridOverlaysInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        grid_node_highlight_configs: list[Overlay.GridNodeHighlightConfig]
    ):
        super().__init__(
            gridNodeHighlightConfigs=grid_node_highlight_configs
        )


class SetShowFlexOverlaysInput(InputModel):

    flexNodeHighlightConfigs: list[Overlay.FlexNodeHighlightConfig]


class SetShowFlexOverlays(CDPMethod[None]):

    INPUT_VALIDATOR = SetShowFlexOverlaysInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        flex_node_highlight_configs: list[Overlay.FlexNodeHighlightConfig]
    ):
        super().__init__(
            flexNodeHighlightConfigs=flex_node_highlight_configs
        )


class SetShowScrollSnapOverlaysInput(InputModel):

    scrollSnapHighlightConfigs: list[Overlay.ScrollSnapHighlightConfig]


class SetShowScrollSnapOverlays(CDPMethod[None]):

    INPUT_VALIDATOR = SetShowScrollSnapOverlaysInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        scroll_snap_highlight_configs: list[Overlay.ScrollSnapHighlightConfig]
    ):
        super().__init__(
            scrollSnapHighlightConfigs=scroll_snap_highlight_configs
        )


class SetShowContainerQueryOverlaysInput(InputModel):

    containerQueryHighlightConfigs: list[Overlay.ContainerQueryHighlightConfig]


class SetShowContainerQueryOverlays(CDPMethod[None]):

    INPUT_VALIDATOR = SetShowContainerQueryOverlaysInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        container_query_highlight_configs: list[Overlay.ContainerQueryHighlightConfig]
    ):
        super().__init__(
            containerQueryHighlightConfigs=container_query_highlight_configs
        )


class SetShowPaintRectsInput(InputModel):

    result: bool


class SetShowPaintRects(CDPMethod[None]):
    """ Requests that backend shows paint rectangles """

    INPUT_VALIDATOR = SetShowPaintRectsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        result: bool
    ):
        super().__init__(
            result=result
        )


class SetShowLayoutShiftRegionsInput(InputModel):

    result: bool


class SetShowLayoutShiftRegions(CDPMethod[None]):
    """ Requests that backend shows layout shift regions """

    INPUT_VALIDATOR = SetShowLayoutShiftRegionsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        result: bool
    ):
        super().__init__(
            result=result
        )


class SetShowScrollBottleneckRectsInput(InputModel):

    show: bool


class SetShowScrollBottleneckRects(CDPMethod[None]):
    """ Requests that backend shows scroll bottleneck rects """

    INPUT_VALIDATOR = SetShowScrollBottleneckRectsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        show: bool
    ):
        super().__init__(
            show=show
        )


class SetShowHitTestBordersInput(InputModel):

    show: bool


class SetShowHitTestBorders(CDPMethod[None]):  # deprecated
    """ Deprecated, no longer has any effect. """

    INPUT_VALIDATOR = SetShowHitTestBordersInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        show: bool
    ):
        super().__init__(
            show=show
        )


class SetShowWebVitalsInput(InputModel):

    show: bool


class SetShowWebVitals(CDPMethod[None]):  # deprecated
    """ Deprecated, no longer has any effect. """

    INPUT_VALIDATOR = SetShowWebVitalsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        show: bool
    ):
        super().__init__(
            show=show
        )


class SetShowViewportSizeOnResizeInput(InputModel):

    show: bool


class SetShowViewportSizeOnResize(CDPMethod[None]):
    """ Paints viewport size upon main frame resize. """

    INPUT_VALIDATOR = SetShowViewportSizeOnResizeInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        show: bool
    ):
        super().__init__(
            show=show
        )


class SetShowHingeInput(InputModel):

    hingeConfig: Overlay.HingeConfig | None = None


class SetShowHinge(CDPMethod[None]):
    """ Add a dual screen device hinge """

    INPUT_VALIDATOR = SetShowHingeInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        hinge_config: Overlay.HingeConfig | None = None
    ):
        super().__init__(
            hingeConfig=hinge_config
        )


class SetShowIsolatedElementsInput(InputModel):

    isolatedElementHighlightConfigs: list[Overlay.IsolatedElementHighlightConfig]


class SetShowIsolatedElements(CDPMethod[None]):
    """ Show elements in isolation mode with overlays. """

    INPUT_VALIDATOR = SetShowIsolatedElementsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        isolated_element_highlight_configs: list[Overlay.IsolatedElementHighlightConfig]
    ):
        super().__init__(
            isolatedElementHighlightConfigs=isolated_element_highlight_configs
        )


class SetShowWindowControlsOverlayInput(InputModel):

    windowControlsOverlayConfig: Overlay.WindowControlsOverlayConfig | None = None


class SetShowWindowControlsOverlay(CDPMethod[None]):
    """ Show Window Controls Overlay for PWA """

    INPUT_VALIDATOR = SetShowWindowControlsOverlayInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        window_controls_overlay_config: Overlay.WindowControlsOverlayConfig | None = None
    ):
        super().__init__(
            windowControlsOverlayConfig=window_controls_overlay_config
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

The Browser domain defines methods and events for browser managing.
https://chromedevtools.github.io/devtools-protocol/tot/Browser/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from typing import Literal

from cdpkit.protocol._types import (
    Browser,
    Target,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class SetPermissionInput(InputModel):

    permission: Browser.PermissionDescriptor
    setting: Browser.PermissionSetting
    origin: str | None = None
    embeddedOrigin: str | None = None
    browserContextId: Browser.BrowserContextID | None = None


class SetPermission(CDPMethod[None]):  # experimental
    """ Set permission settings for given embedding and embedded origins. """

    INPUT_VALIDATOR = SetPermissionInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        permission: Browser.PermissionDescriptor,
        setting: Browser.PermissionSetting,
        origin: str | None = None,
        embedded_origin: str | None = None,
        browser_context_id: Browser.BrowserContextID | None = None
    ):
        super().__init__(
            permission=permission,
            setting=setting,
            origin=origin,
            embeddedOrigin=embedded_origin,
            browserContextId=browser_context_id
        )


class GrantPermissionsInput(InputModel):

    permissions: list[Browser.PermissionType]
    origin: str | None = None
    browserContextId: Browser.BrowserContextID | None = None


class GrantPermissions(CDPMethod[None]):  # experimental deprecated
    """ Grant specific permissions to the given origin and reject all others. Deprecated. Use
    setPermission instead. """

    INPUT_VALIDATOR = GrantPermissionsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        permissions: list[Browser.PermissionType],
        origin: str | None = None,
        browser_context_id: Browser.BrowserContextID | None = None
    ):
        super().__init__(
            permissions=permissions,
            origin=origin,
            browserContextId=browser_context_id
        )


class ResetPermissionsInput(InputModel):

    browserContextId: Browser.BrowserContextID | None = None


class ResetPermissions(CDPMethod[None]):
    """ Reset all permission management for all origins. """

    INPUT_VALIDATOR = ResetPermissionsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        browser_context_id: Browser.BrowserContextID | None = None
    ):
        super().__init__(
            browserContextId=browser_context_id
        )


class SetDownloadBehaviorInput(InputModel):

    behavior: Literal['deny', 'allow', 'allowAndName', 'default']
    browserContextId: Browser.BrowserContextID | None = None
    downloadPath: str | None = None
    eventsEnabled: bool | None = None


class SetDownloadBehavior(CDPMethod[None]):  # experimental
    """ Set the behavior when downloading a file. """

    INPUT_VALIDATOR = SetDownloadBehaviorInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        behavior: Literal['deny', 'allow', 'allowAndName', 'default'],
        browser_context_id: Browser.BrowserContextID | None = None,
        download_path: str | None = None,
        events_enabled: bool | None = None
    ):
        super().__init__(
            behavior=behavior,
            browserContextId=browser_context_id,
            downloadPath=download_path,
            eventsEnabled=events_enabled
        )


class CancelDownloadInput(InputModel):

    guid: str
    browserContextId: Browser.BrowserContextID | None = None


class CancelDownload(CDPMethod[None]):  # experimental
    """ Cancel a download if in progress """

    INPUT_VALIDATOR = CancelDownloadInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        guid: str,
        browser_context_id: Browser.BrowserContextID | None = None
    ):
        super().__init__(
            guid=guid,
            browserContextId=browser_context_id
        )


class Close(CDPMethod[None]):
    """ Close browser gracefully. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Crash(CDPMethod[None]):  # experimental
    """ Crashes browser on the main thread. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class CrashGpuProcess(CDPMethod[None]):  # experimental
    """ Crashes GPU process. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class GetVersionOutput(OutputModel):

    protocolVersion: str
    product: str
    revision: str
    userAgent: str
    jsVersion: str


class GetVersion(CDPMethod[GetVersionOutput]):
    """ Returns version information. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetVersionOutput


class GetBrowserCommandLineOutput(OutputModel):

    arguments: list[str]


class GetBrowserCommandLine(CDPMethod[GetBrowserCommandLineOutput]):  # experimental
    """ Returns the command line switches for the browser process if, and only if
    --enable-automation is on the commandline. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetBrowserCommandLineOutput


class GetHistogramsInput(InputModel):

    query: str | None = None
    delta: bool | None = None


class GetHistogramsOutput(OutputModel):

    histograms: list[Browser.Histogram]


class GetHistograms(CDPMethod[GetHistogramsOutput]):  # experimental
    """ Get Chrome histograms. """

    INPUT_VALIDATOR = GetHistogramsInput
    OUTPUT_VALIDATOR = GetHistogramsOutput

    def __init__(
        self,
        *,
        query: str | None = None,
        delta: bool | None = None
    ):
        super().__init__(
            query=query,
            delta=delta
        )


class GetHistogramInput(InputModel):

    name: str
    delta: bool | None = None


class GetHistogramOutput(OutputModel):

    histogram: Browser.Histogram


class GetHistogram(CDPMethod[GetHistogramOutput]):  # experimental
    """ Get a Chrome histogram by name. """

    INPUT_VALIDATOR = GetHistogramInput
    OUTPUT_VALIDATOR = GetHistogramOutput

    def __init__(
        self,
        *,
        name: str,
        delta: bool | None = None
    ):
        super().__init__(
            name=name,
            delta=delta
        )


class GetWindowBoundsInput(InputModel):

    windowId: Browser.WindowID


class GetWindowBoundsOutput(OutputModel):

    bounds: Browser.Bounds


class GetWindowBounds(CDPMethod[GetWindowBoundsOutput]):  # experimental
    """ Get position and size of the browser window. """

    INPUT_VALIDATOR = GetWindowBoundsInput
    OUTPUT_VALIDATOR = GetWindowBoundsOutput

    def __init__(
        self,
        *,
        window_id: Browser.WindowID
    ):
        super().__init__(
            windowId=window_id
        )


class GetWindowForTargetInput(InputModel):

    targetId: Target.TargetID | None = None


class GetWindowForTargetOutput(OutputModel):

    windowId: Browser.WindowID
    bounds: Browser.Bounds


class GetWindowForTarget(CDPMethod[GetWindowForTargetOutput]):  # experimental
    """ Get the browser window that contains the devtools target. """

    INPUT_VALIDATOR = GetWindowForTargetInput
    OUTPUT_VALIDATOR = GetWindowForTargetOutput

    def __init__(
        self,
        *,
        target_id: Target.TargetID | None = None
    ):
        super().__init__(
            targetId=target_id
        )


class SetWindowBoundsInput(InputModel):

    windowId: Browser.WindowID
    bounds: Browser.Bounds


class SetWindowBounds(CDPMethod[None]):  # experimental
    """ Set position and/or size of the browser window. """

    INPUT_VALIDATOR = SetWindowBoundsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        window_id: Browser.WindowID,
        bounds: Browser.Bounds
    ):
        super().__init__(
            windowId=window_id,
            bounds=bounds
        )


class SetContentsSizeInput(InputModel):

    windowId: Browser.WindowID
    width: int | None = None
    height: int | None = None


class SetContentsSize(CDPMethod[None]):  # experimental
    """ Set size of the browser contents resizing browser window as necessary. """

    INPUT_VALIDATOR = SetContentsSizeInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        window_id: Browser.WindowID,
        width: int | None = None,
        height: int | None = None
    ):
        super().__init__(
            windowId=window_id,
            width=width,
            height=height
        )


class SetDockTileInput(InputModel):

    badgeLabel: str | None = None
    image: str | None = None


class SetDockTile(CDPMethod[None]):  # experimental
    """ Set dock tile details, platform-specific. """

    INPUT_VALIDATOR = SetDockTileInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        badge_label: str | None = None,
        image: str | None = None
    ):
        super().__init__(
            badgeLabel=badge_label,
            image=image
        )


class ExecuteBrowserCommandInput(InputModel):

    commandId: Browser.BrowserCommandId


class ExecuteBrowserCommand(CDPMethod[None]):  # experimental
    """ Invoke custom browser commands used by telemetry. """

    INPUT_VALIDATOR = ExecuteBrowserCommandInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        command_id: Browser.BrowserCommandId
    ):
        super().__init__(
            commandId=command_id
        )


class AddPrivacySandboxEnrollmentOverrideInput(InputModel):

    url: str


class AddPrivacySandboxEnrollmentOverride(CDPMethod[None]):
    """ Allows a site to use privacy sandbox features that require enrollment
    without the site actually being enrolled. Only supported on page targets. """

    INPUT_VALIDATOR = AddPrivacySandboxEnrollmentOverrideInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        url: str
    ):
        super().__init__(
            url=url
        )


class AddPrivacySandboxCoordinatorKeyConfigInput(InputModel):

    api: Browser.PrivacySandboxAPI
    coordinatorOrigin: str
    keyConfig: str
    browserContextId: Browser.BrowserContextID | None = None


class AddPrivacySandboxCoordinatorKeyConfig(CDPMethod[None]):
    """ Configures encryption keys used with a given privacy sandbox API to talk
    to a trusted coordinator.  Since this is intended for test automation only,
    coordinatorOrigin must be a .test domain. No existing coordinator
    configuration for the origin may exist. """

    INPUT_VALIDATOR = AddPrivacySandboxCoordinatorKeyConfigInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        api: Browser.PrivacySandboxAPI,
        coordinator_origin: str,
        key_config: str,
        browser_context_id: Browser.BrowserContextID | None = None
    ):
        super().__init__(
            api=api,
            coordinatorOrigin=coordinator_origin,
            keyConfig=key_config,
            browserContextId=browser_context_id
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/Inspector/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol.base import CDPEvent


class Detached(CDPEvent):
    """ Fired when remote debugging connection is about to be terminated. Contains detach reason. """

    reason: str


class TargetCrashed(CDPEvent):
    """ Fired when debugging target has crashed """

    ...


class TargetReloadedAfterCrash(CDPEvent):
    """ Fired when debugging target has reloaded after crash """

    ...


class WorkerScriptLoaded(CDPEvent):
    """ Fired on worker targets when main worker script and any imported scripts have been evaluated. """

    ...



from .base import CustomException


class InvalidCallback(CustomException):
    ERROR_INFO = 'Invalid callback, must be callable.'


class CallbackParameterError(CustomException):
    ERROR_INFO = 'Callback parameter error.'


class NetworkError(CustomException):
    ERROR_INFO = 'Network error.'


class InvalidResponse(CustomException):
    ERROR_INFO = 'The response received is invalid.'


class CommandExecutionTimeout(CustomException):
    ERROR_INFO = 'The command execution timed out.'


class CommandExecutionError(CustomException):
    ERROR_INFO = 'The command execution error.'


class WebSocketConnectionClosed(CustomException):
    ERROR_INFO = 'The WebSocket connection is closed'



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain allows configuring virtual authenticators to test the WebAuthn
API.
https://chromedevtools.github.io/devtools-protocol/tot/WebAuthn/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    WebAuthn,
)
from cdpkit.protocol.base import CDPEvent


class CredentialAdded(CDPEvent):
    """ Triggered when a credential is added to an authenticator. """

    authenticatorId: WebAuthn.AuthenticatorId
    credential: WebAuthn.Credential


class CredentialDeleted(CDPEvent):
    """ Triggered when a credential is deleted, e.g. through
    PublicKeyCredential.signalUnknownCredential(). """

    authenticatorId: WebAuthn.AuthenticatorId
    credentialId: str


class CredentialUpdated(CDPEvent):
    """ Triggered when a credential is updated, e.g. through
    PublicKeyCredential.signalCurrentUserDetails(). """

    authenticatorId: WebAuthn.AuthenticatorId
    credential: WebAuthn.Credential


class CredentialAsserted(CDPEvent):
    """ Triggered when a credential is used in a webauthn assertion. """

    authenticatorId: WebAuthn.AuthenticatorId
    credential: WebAuthn.Credential



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Network domain allows tracking network activities of the page. It exposes information about http,
file, data and other requests and responses, their headers, bodies, timing, etc.
https://chromedevtools.github.io/devtools-protocol/tot/Network/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    NetworkAlternateProtocolUsage,
    NetworkAssociatedCookie,
    NetworkAuthChallenge,
    NetworkAuthChallengeResponse,
    NetworkBlockedReason,
    NetworkBlockedSetCookieWithReason,
    NetworkBlockPattern,
    NetworkCachedResource,
    NetworkCertificateTransparencyCompliance,
    NetworkClientSecurityState,
    NetworkConnectionType,
    NetworkConnectTiming,
    NetworkContentEncoding,
    NetworkContentSecurityPolicySource,
    NetworkContentSecurityPolicyStatus,
    NetworkCookie,
    NetworkCookieBlockedReason,
    NetworkCookieExemptionReason,
    NetworkCookieParam,
    NetworkCookiePartitionKey,
    NetworkCookiePriority,
    NetworkCookieSameSite,
    NetworkCookieSourceScheme,
    NetworkCorsError,
    NetworkCorsErrorStatus,
    NetworkCrossOriginEmbedderPolicyStatus,
    NetworkCrossOriginEmbedderPolicyValue,
    NetworkCrossOriginOpenerPolicyStatus,
    NetworkCrossOriginOpenerPolicyValue,
    NetworkDirectSocketDnsQueryType,
    NetworkDirectTCPSocketOptions,
    NetworkDirectUDPMessage,
    NetworkDirectUDPSocketOptions,
    NetworkErrorReason,
    NetworkExemptedSetCookieWithReason,
    NetworkHeaders,
    NetworkInitiator,
    NetworkInterceptionId,
    NetworkInterceptionStage,
    NetworkIPAddressSpace,
    NetworkLoaderId,
    NetworkLoadNetworkResourceOptions,
    NetworkLoadNetworkResourcePageResult,
    NetworkMonotonicTime,
    NetworkNetworkConditions,
    NetworkPostDataEntry,
    NetworkPrivateNetworkRequestPolicy,
    NetworkRenderBlockingBehavior,
    NetworkReportId,
    NetworkReportingApiEndpoint,
    NetworkReportingApiReport,
    NetworkReportStatus,
    NetworkRequest,
    NetworkRequestId,
    NetworkRequestPattern,
    NetworkResourcePriority,
    NetworkResourceTiming,
    NetworkResourceType,
    NetworkResponse,
    NetworkSecurityDetails,
    NetworkSecurityIsolationStatus,
    NetworkServiceWorkerResponseSource,
    NetworkServiceWorkerRouterInfo,
    NetworkServiceWorkerRouterSource,
    NetworkSetCookieBlockedReason,
    NetworkSignedCertificateTimestamp,
    NetworkSignedExchangeError,
    NetworkSignedExchangeErrorField,
    NetworkSignedExchangeHeader,
    NetworkSignedExchangeInfo,
    NetworkSignedExchangeSignature,
    NetworkTimeSinceEpoch,
    NetworkTrustTokenOperationType,
    NetworkTrustTokenParams,
    NetworkWebSocketFrame,
    NetworkWebSocketRequest,
    NetworkWebSocketResponse,
)

ResourceType = NetworkResourceType
LoaderId = NetworkLoaderId
RequestId = NetworkRequestId
InterceptionId = NetworkInterceptionId
ErrorReason = NetworkErrorReason
TimeSinceEpoch = NetworkTimeSinceEpoch
MonotonicTime = NetworkMonotonicTime
Headers = NetworkHeaders
ConnectionType = NetworkConnectionType
CookieSameSite = NetworkCookieSameSite
CookiePriority = NetworkCookiePriority
CookieSourceScheme = NetworkCookieSourceScheme
ResourceTiming = NetworkResourceTiming
ResourcePriority = NetworkResourcePriority
RenderBlockingBehavior = NetworkRenderBlockingBehavior
PostDataEntry = NetworkPostDataEntry
Request = NetworkRequest
SignedCertificateTimestamp = NetworkSignedCertificateTimestamp
SecurityDetails = NetworkSecurityDetails
CertificateTransparencyCompliance = NetworkCertificateTransparencyCompliance
BlockedReason = NetworkBlockedReason
CorsError = NetworkCorsError
CorsErrorStatus = NetworkCorsErrorStatus
ServiceWorkerResponseSource = NetworkServiceWorkerResponseSource
TrustTokenParams = NetworkTrustTokenParams
TrustTokenOperationType = NetworkTrustTokenOperationType
AlternateProtocolUsage = NetworkAlternateProtocolUsage
ServiceWorkerRouterSource = NetworkServiceWorkerRouterSource
ServiceWorkerRouterInfo = NetworkServiceWorkerRouterInfo
Response = NetworkResponse
WebSocketRequest = NetworkWebSocketRequest
WebSocketResponse = NetworkWebSocketResponse
WebSocketFrame = NetworkWebSocketFrame
CachedResource = NetworkCachedResource
Initiator = NetworkInitiator
CookiePartitionKey = NetworkCookiePartitionKey
Cookie = NetworkCookie
SetCookieBlockedReason = NetworkSetCookieBlockedReason
CookieBlockedReason = NetworkCookieBlockedReason
CookieExemptionReason = NetworkCookieExemptionReason
BlockedSetCookieWithReason = NetworkBlockedSetCookieWithReason
ExemptedSetCookieWithReason = NetworkExemptedSetCookieWithReason
AssociatedCookie = NetworkAssociatedCookie
CookieParam = NetworkCookieParam
AuthChallenge = NetworkAuthChallenge
AuthChallengeResponse = NetworkAuthChallengeResponse
InterceptionStage = NetworkInterceptionStage
RequestPattern = NetworkRequestPattern
SignedExchangeSignature = NetworkSignedExchangeSignature
SignedExchangeHeader = NetworkSignedExchangeHeader
SignedExchangeErrorField = NetworkSignedExchangeErrorField
SignedExchangeError = NetworkSignedExchangeError
SignedExchangeInfo = NetworkSignedExchangeInfo
ContentEncoding = NetworkContentEncoding
NetworkConditions = NetworkNetworkConditions
BlockPattern = NetworkBlockPattern
DirectSocketDnsQueryType = NetworkDirectSocketDnsQueryType
DirectTCPSocketOptions = NetworkDirectTCPSocketOptions
DirectUDPSocketOptions = NetworkDirectUDPSocketOptions
DirectUDPMessage = NetworkDirectUDPMessage
PrivateNetworkRequestPolicy = NetworkPrivateNetworkRequestPolicy
IPAddressSpace = NetworkIPAddressSpace
ConnectTiming = NetworkConnectTiming
ClientSecurityState = NetworkClientSecurityState
CrossOriginOpenerPolicyValue = NetworkCrossOriginOpenerPolicyValue
CrossOriginOpenerPolicyStatus = NetworkCrossOriginOpenerPolicyStatus
CrossOriginEmbedderPolicyValue = NetworkCrossOriginEmbedderPolicyValue
CrossOriginEmbedderPolicyStatus = NetworkCrossOriginEmbedderPolicyStatus
ContentSecurityPolicySource = NetworkContentSecurityPolicySource
ContentSecurityPolicyStatus = NetworkContentSecurityPolicyStatus
SecurityIsolationStatus = NetworkSecurityIsolationStatus
ReportStatus = NetworkReportStatus
ReportId = NetworkReportId
ReportingApiReport = NetworkReportingApiReport
ReportingApiEndpoint = NetworkReportingApiEndpoint
LoadNetworkResourcePageResult = NetworkLoadNetworkResourcePageResult
LoadNetworkResourceOptions = NetworkLoadNetworkResourceOptions



from .events import CredentialAdded, CredentialAsserted, CredentialDeleted, CredentialUpdated
from .methods import (
    AddCredential,
    AddVirtualAuthenticator,
    ClearCredentials,
    Disable,
    Enable,
    GetCredential,
    GetCredentials,
    RemoveCredential,
    RemoveVirtualAuthenticator,
    SetAutomaticPresenceSimulation,
    SetCredentialProperties,
    SetResponseOverrideBits,
    SetUserVerified,
)
from .types import (
    AuthenticatorId,
    AuthenticatorProtocol,
    AuthenticatorTransport,
    Credential,
    Ctap2Version,
    VirtualAuthenticatorOptions,
)

__all__ = [
    AuthenticatorId,
    AuthenticatorProtocol,
    Ctap2Version,
    AuthenticatorTransport,
    VirtualAuthenticatorOptions,
    Credential,
    CredentialAdded,
    CredentialDeleted,
    CredentialUpdated,
    CredentialAsserted,
    Enable,
    Disable,
    AddVirtualAuthenticator,
    SetResponseOverrideBits,
    RemoveVirtualAuthenticator,
    AddCredential,
    GetCredential,
    GetCredentials,
    RemoveCredential,
    ClearCredentials,
    SetUserVerified,
    SetAutomaticPresenceSimulation,
    SetCredentialProperties,
]



from .events import (
    AudioListenerCreated,
    AudioListenerWillBeDestroyed,
    AudioNodeCreated,
    AudioNodeWillBeDestroyed,
    AudioParamCreated,
    AudioParamWillBeDestroyed,
    ContextChanged,
    ContextCreated,
    ContextWillBeDestroyed,
    NodeParamConnected,
    NodeParamDisconnected,
    NodesConnected,
    NodesDisconnected,
)
from .methods import Disable, Enable, GetRealtimeData
from .types import (
    AudioListener,
    AudioNode,
    AudioParam,
    AutomationRate,
    BaseAudioContext,
    ChannelCountMode,
    ChannelInterpretation,
    ContextRealtimeData,
    ContextState,
    ContextType,
    GraphObjectId,
    NodeType,
    ParamType,
)

__all__ = [
    GraphObjectId,
    ContextType,
    ContextState,
    NodeType,
    ChannelCountMode,
    ChannelInterpretation,
    ParamType,
    AutomationRate,
    ContextRealtimeData,
    BaseAudioContext,
    AudioListener,
    AudioNode,
    AudioParam,
    ContextCreated,
    ContextWillBeDestroyed,
    ContextChanged,
    AudioListenerCreated,
    AudioListenerWillBeDestroyed,
    AudioNodeCreated,
    AudioNodeWillBeDestroyed,
    AudioParamCreated,
    AudioParamWillBeDestroyed,
    NodesConnected,
    NodesDisconnected,
    NodeParamConnected,
    NodeParamDisconnected,
    Enable,
    Disable,
    GetRealtimeData,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

The SystemInfo domain defines methods and events for querying low-level system information.
https://chromedevtools.github.io/devtools-protocol/tot/SystemInfo/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    SystemInfo,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class GetInfoOutput(OutputModel):

    gpu: SystemInfo.GPUInfo
    modelName: str
    modelVersion: str
    commandLine: str


class GetInfo(CDPMethod[GetInfoOutput]):
    """ Returns information about the system. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetInfoOutput


class GetFeatureStateInput(InputModel):

    featureState: str


class GetFeatureStateOutput(OutputModel):

    featureEnabled: bool


class GetFeatureState(CDPMethod[GetFeatureStateOutput]):
    """ Returns information about the feature state. """

    INPUT_VALIDATOR = GetFeatureStateInput
    OUTPUT_VALIDATOR = GetFeatureStateOutput

    def __init__(
        self,
        *,
        feature_state: str
    ):
        super().__init__(
            featureState=feature_state
        )


class GetProcessInfoOutput(OutputModel):

    processInfo: list[SystemInfo.ProcessInfo]


class GetProcessInfo(CDPMethod[GetProcessInfoOutput]):
    """ Returns information about all running processes. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetProcessInfoOutput



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

The Browser domain defines methods and events for browser managing.
https://chromedevtools.github.io/devtools-protocol/tot/Browser/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    BrowserBounds,
    BrowserBrowserCommandId,
    BrowserBrowserContextID,
    BrowserBucket,
    BrowserHistogram,
    BrowserPermissionDescriptor,
    BrowserPermissionSetting,
    BrowserPermissionType,
    BrowserPrivacySandboxAPI,
    BrowserWindowID,
    BrowserWindowState,
)

BrowserContextID = BrowserBrowserContextID
WindowID = BrowserWindowID
WindowState = BrowserWindowState
Bounds = BrowserBounds
PermissionType = BrowserPermissionType
PermissionSetting = BrowserPermissionSetting
PermissionDescriptor = BrowserPermissionDescriptor
BrowserCommandId = BrowserBrowserCommandId
Bucket = BrowserBucket
Histogram = BrowserHistogram
PrivacySandboxAPI = BrowserPrivacySandboxAPI



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Defines commands and events for Autofill.
https://chromedevtools.github.io/devtools-protocol/tot/Autofill/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    DOM,
    Autofill,
    Page,
)
from cdpkit.protocol.base import CDPMethod, InputModel


class TriggerInput(InputModel):

    fieldId: DOM.BackendNodeId
    frameId: Page.FrameId | None = None
    card: Autofill.CreditCard | None = None
    address: Autofill.Address | None = None


class Trigger(CDPMethod[None]):
    """ Trigger autofill on a form identified by the fieldId.
    If the field and related form cannot be autofilled, returns an error. """

    INPUT_VALIDATOR = TriggerInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        field_id: DOM.BackendNodeId,
        frame_id: Page.FrameId | None = None,
        card: Autofill.CreditCard | None = None,
        address: Autofill.Address | None = None
    ):
        super().__init__(
            fieldId=field_id,
            frameId=frame_id,
            card=card,
            address=address
        )


class SetAddressesInput(InputModel):

    addresses: list[Autofill.Address]


class SetAddresses(CDPMethod[None]):
    """ Set addresses so that developers can verify their forms implementation. """

    INPUT_VALIDATOR = SetAddressesInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        addresses: list[Autofill.Address]
    ):
        super().__init__(
            addresses=addresses
        )


class Disable(CDPMethod[None]):
    """ Disables autofill domain notifications. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables autofill domain notifications. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/CacheStorage/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    CacheStorage,
    Storage,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class DeleteCacheInput(InputModel):

    cacheId: CacheStorage.CacheId


class DeleteCache(CDPMethod[None]):
    """ Deletes a cache. """

    INPUT_VALIDATOR = DeleteCacheInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        cache_id: CacheStorage.CacheId
    ):
        super().__init__(
            cacheId=cache_id
        )


class DeleteEntryInput(InputModel):

    cacheId: CacheStorage.CacheId
    request: str


class DeleteEntry(CDPMethod[None]):
    """ Deletes a cache entry. """

    INPUT_VALIDATOR = DeleteEntryInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        cache_id: CacheStorage.CacheId,
        request: str
    ):
        super().__init__(
            cacheId=cache_id,
            request=request
        )


class RequestCacheNamesInput(InputModel):

    securityOrigin: str | None = None
    storageKey: str | None = None
    storageBucket: Storage.StorageBucket | None = None


class RequestCacheNamesOutput(OutputModel):

    caches: list[CacheStorage.Cache]


class RequestCacheNames(CDPMethod[RequestCacheNamesOutput]):
    """ Requests cache names. """

    INPUT_VALIDATOR = RequestCacheNamesInput
    OUTPUT_VALIDATOR = RequestCacheNamesOutput

    def __init__(
        self,
        *,
        security_origin: str | None = None,
        storage_key: str | None = None,
        storage_bucket: Storage.StorageBucket | None = None
    ):
        super().__init__(
            securityOrigin=security_origin,
            storageKey=storage_key,
            storageBucket=storage_bucket
        )


class RequestCachedResponseInput(InputModel):

    cacheId: CacheStorage.CacheId
    requestURL: str
    requestHeaders: list[CacheStorage.Header]


class RequestCachedResponseOutput(OutputModel):

    response: CacheStorage.CachedResponse


class RequestCachedResponse(CDPMethod[RequestCachedResponseOutput]):
    """ Fetches cache entry. """

    INPUT_VALIDATOR = RequestCachedResponseInput
    OUTPUT_VALIDATOR = RequestCachedResponseOutput

    def __init__(
        self,
        *,
        cache_id: CacheStorage.CacheId,
        request_url: str,
        request_headers: list[CacheStorage.Header]
    ):
        super().__init__(
            cacheId=cache_id,
            requestURL=request_url,
            requestHeaders=request_headers
        )


class RequestEntriesInput(InputModel):

    cacheId: CacheStorage.CacheId
    skipCount: int | None = None
    pageSize: int | None = None
    pathFilter: str | None = None


class RequestEntriesOutput(OutputModel):

    cacheDataEntries: list[CacheStorage.DataEntry]
    returnCount: float


class RequestEntries(CDPMethod[RequestEntriesOutput]):
    """ Requests data from cache. """

    INPUT_VALIDATOR = RequestEntriesInput
    OUTPUT_VALIDATOR = RequestEntriesOutput

    def __init__(
        self,
        *,
        cache_id: CacheStorage.CacheId,
        skip_count: int | None = None,
        page_size: int | None = None,
        path_filter: str | None = None
    ):
        super().__init__(
            cacheId=cache_id,
            skipCount=skip_count,
            pageSize=page_size,
            pathFilter=path_filter
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Provides access to log entries.
https://chromedevtools.github.io/devtools-protocol/tot/Log/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    LogLogEntry,
    LogViolationSetting,
)

LogEntry = LogLogEntry
ViolationSetting = LogViolationSetting



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/FileSystem/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    FileSystemBucketFileSystemLocator,
    FileSystemDirectory,
    FileSystemFile,
)

File = FileSystemFile
Directory = FileSystemDirectory
BucketFileSystemLocator = FileSystemBucketFileSystemLocator



from .events import DragIntercepted
from .methods import (
    CancelDragging,
    DispatchDragEvent,
    DispatchKeyEvent,
    DispatchMouseEvent,
    DispatchTouchEvent,
    EmulateTouchFromMouseEvent,
    ImeSetComposition,
    InsertText,
    SetIgnoreInputEvents,
    SetInterceptDrags,
    SynthesizePinchGesture,
    SynthesizeScrollGesture,
    SynthesizeTapGesture,
)
from .types import DragData, DragDataItem, GestureSourceType, MouseButton, TimeSinceEpoch, TouchPoint

__all__ = [
    TouchPoint,
    GestureSourceType,
    MouseButton,
    TimeSinceEpoch,
    DragDataItem,
    DragData,
    DragIntercepted,
    DispatchDragEvent,
    DispatchKeyEvent,
    InsertText,
    ImeSetComposition,
    DispatchMouseEvent,
    DispatchTouchEvent,
    CancelDragging,
    EmulateTouchFromMouseEvent,
    SetIgnoreInputEvents,
    SetInterceptDrags,
    SynthesizePinchGesture,
    SynthesizeScrollGesture,
    SynthesizeTapGesture,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain exposes DOM read/write operations. Each DOM Node is represented with its mirror object
that has an `id`. This `id` can be used to get additional information on the Node, resolve it into
the JavaScript object wrapper, etc. It is important that client receives DOM events only for the
nodes that are known to the client. Backend keeps track of the nodes that were sent to the client
and never sends the same node twice. It is client's responsibility to collect information about
the nodes that were sent to the client. Note that `iframe` owner elements will return
corresponding document elements as their child nodes.
https://chromedevtools.github.io/devtools-protocol/tot/DOM/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from typing import Literal

from cdpkit.protocol._types import (
    DOM,
    Page,
    Runtime,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class CollectClassNamesFromSubtreeInput(InputModel):

    nodeId: DOM.NodeId


class CollectClassNamesFromSubtreeOutput(OutputModel):

    classNames: list[str]


class CollectClassNamesFromSubtree(CDPMethod[CollectClassNamesFromSubtreeOutput]):  # experimental
    """ Collects class names for the node with given id and all of it's child nodes. """

    INPUT_VALIDATOR = CollectClassNamesFromSubtreeInput
    OUTPUT_VALIDATOR = CollectClassNamesFromSubtreeOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId
    ):
        super().__init__(
            nodeId=node_id
        )


class CopyToInput(InputModel):

    nodeId: DOM.NodeId
    targetNodeId: DOM.NodeId
    insertBeforeNodeId: DOM.NodeId | None = None


class CopyToOutput(OutputModel):

    nodeId: DOM.NodeId


class CopyTo(CDPMethod[CopyToOutput]):  # experimental
    """ Creates a deep copy of the specified node and places it into the target container before the
    given anchor. """

    INPUT_VALIDATOR = CopyToInput
    OUTPUT_VALIDATOR = CopyToOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        target_node_id: DOM.NodeId,
        insert_before_node_id: DOM.NodeId | None = None
    ):
        super().__init__(
            nodeId=node_id,
            targetNodeId=target_node_id,
            insertBeforeNodeId=insert_before_node_id
        )


class DescribeNodeInput(InputModel):

    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectId: Runtime.RemoteObjectId | None = None
    depth: int | None = None
    pierce: bool | None = None


class DescribeNodeOutput(OutputModel):

    node: DOM.Node


class DescribeNode(CDPMethod[DescribeNodeOutput]):
    """ Describes node given its id, does not require domain to be enabled. Does not start tracking any
    objects, can be used for automation. """

    INPUT_VALIDATOR = DescribeNodeInput
    OUTPUT_VALIDATOR = DescribeNodeOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_id: Runtime.RemoteObjectId | None = None,
        depth: int | None = None,
        pierce: bool | None = None
    ):
        super().__init__(
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectId=object_id,
            depth=depth,
            pierce=pierce
        )


class ScrollIntoViewIfNeededInput(InputModel):

    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectId: Runtime.RemoteObjectId | None = None
    rect: DOM.Rect | None = None


class ScrollIntoViewIfNeeded(CDPMethod[None]):
    """ Scrolls the specified rect of the given node into view if not already visible.
    Note: exactly one between nodeId, backendNodeId and objectId should be passed
    to identify the node. """

    INPUT_VALIDATOR = ScrollIntoViewIfNeededInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_id: Runtime.RemoteObjectId | None = None,
        rect: DOM.Rect | None = None
    ):
        super().__init__(
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectId=object_id,
            rect=rect
        )


class Disable(CDPMethod[None]):
    """ Disables DOM agent for the given page. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class DiscardSearchResultsInput(InputModel):

    searchId: str


class DiscardSearchResults(CDPMethod[None]):  # experimental
    """ Discards search results from the session with the given id. `getSearchResults` should no longer
    be called for that search. """

    INPUT_VALIDATOR = DiscardSearchResultsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        search_id: str
    ):
        super().__init__(
            searchId=search_id
        )


class EnableInput(InputModel):

    includeWhitespace: Literal['none', 'all'] | None = None  # experimental


class Enable(CDPMethod[None]):
    """ Enables DOM agent for the given page. """

    INPUT_VALIDATOR = EnableInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        include_whitespace: Literal['none', 'all'] | None = None
    ):
        super().__init__(
            includeWhitespace=include_whitespace
        )


class FocusInput(InputModel):

    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectId: Runtime.RemoteObjectId | None = None


class Focus(CDPMethod[None]):
    """ Focuses the given element. """

    INPUT_VALIDATOR = FocusInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_id: Runtime.RemoteObjectId | None = None
    ):
        super().__init__(
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectId=object_id
        )


class GetAttributesInput(InputModel):

    nodeId: DOM.NodeId


class GetAttributesOutput(OutputModel):

    attributes: list[str]


class GetAttributes(CDPMethod[GetAttributesOutput]):
    """ Returns attributes for the specified node. """

    INPUT_VALIDATOR = GetAttributesInput
    OUTPUT_VALIDATOR = GetAttributesOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId
    ):
        super().__init__(
            nodeId=node_id
        )


class GetBoxModelInput(InputModel):

    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectId: Runtime.RemoteObjectId | None = None


class GetBoxModelOutput(OutputModel):

    model: DOM.BoxModel


class GetBoxModel(CDPMethod[GetBoxModelOutput]):
    """ Returns boxes for the given node. """

    INPUT_VALIDATOR = GetBoxModelInput
    OUTPUT_VALIDATOR = GetBoxModelOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_id: Runtime.RemoteObjectId | None = None
    ):
        super().__init__(
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectId=object_id
        )


class GetContentQuadsInput(InputModel):

    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectId: Runtime.RemoteObjectId | None = None


class GetContentQuadsOutput(OutputModel):

    quads: list[DOM.Quad]


class GetContentQuads(CDPMethod[GetContentQuadsOutput]):  # experimental
    """ Returns quads that describe node position on the page. This method
    might return multiple quads for inline nodes. """

    INPUT_VALIDATOR = GetContentQuadsInput
    OUTPUT_VALIDATOR = GetContentQuadsOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_id: Runtime.RemoteObjectId | None = None
    ):
        super().__init__(
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectId=object_id
        )


class GetDocumentInput(InputModel):

    depth: int | None = None
    pierce: bool | None = None


class GetDocumentOutput(OutputModel):

    root: DOM.Node


class GetDocument(CDPMethod[GetDocumentOutput]):
    """ Returns the root DOM node (and optionally the subtree) to the caller.
    Implicitly enables the DOM domain events for the current target. """

    INPUT_VALIDATOR = GetDocumentInput
    OUTPUT_VALIDATOR = GetDocumentOutput

    def __init__(
        self,
        *,
        depth: int | None = None,
        pierce: bool | None = None
    ):
        super().__init__(
            depth=depth,
            pierce=pierce
        )


class GetFlattenedDocumentInput(InputModel):

    depth: int | None = None
    pierce: bool | None = None


class GetFlattenedDocumentOutput(OutputModel):

    nodes: list[DOM.Node]


class GetFlattenedDocument(CDPMethod[GetFlattenedDocumentOutput]):  # deprecated
    """ Returns the root DOM node (and optionally the subtree) to the caller.
    Deprecated, as it is not designed to work well with the rest of the DOM agent.
    Use DOMSnapshot.captureSnapshot instead. """

    INPUT_VALIDATOR = GetFlattenedDocumentInput
    OUTPUT_VALIDATOR = GetFlattenedDocumentOutput

    def __init__(
        self,
        *,
        depth: int | None = None,
        pierce: bool | None = None
    ):
        super().__init__(
            depth=depth,
            pierce=pierce
        )


class GetNodesForSubtreeByStyleInput(InputModel):

    nodeId: DOM.NodeId
    computedStyles: list[DOM.CSSComputedStyleProperty]
    pierce: bool | None = None


class GetNodesForSubtreeByStyleOutput(OutputModel):

    nodeIds: list[DOM.NodeId]


class GetNodesForSubtreeByStyle(CDPMethod[GetNodesForSubtreeByStyleOutput]):  # experimental
    """ Finds nodes with a given computed style in a subtree. """

    INPUT_VALIDATOR = GetNodesForSubtreeByStyleInput
    OUTPUT_VALIDATOR = GetNodesForSubtreeByStyleOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        computed_styles: list[DOM.CSSComputedStyleProperty],
        pierce: bool | None = None
    ):
        super().__init__(
            nodeId=node_id,
            computedStyles=computed_styles,
            pierce=pierce
        )


class GetNodeForLocationInput(InputModel):

    x: int
    y: int
    includeUserAgentShadowDOM: bool | None = None
    ignorePointerEventsNone: bool | None = None


class GetNodeForLocationOutput(OutputModel):

    backendNodeId: DOM.BackendNodeId
    frameId: Page.FrameId
    nodeId: DOM.NodeId | None = None


class GetNodeForLocation(CDPMethod[GetNodeForLocationOutput]):
    """ Returns node id at given location. Depending on whether DOM domain is enabled, nodeId is
    either returned or not. """

    INPUT_VALIDATOR = GetNodeForLocationInput
    OUTPUT_VALIDATOR = GetNodeForLocationOutput

    def __init__(
        self,
        *,
        x: int,
        y: int,
        include_user_agent_shadow_dom: bool | None = None,
        ignore_pointer_events_none: bool | None = None
    ):
        super().__init__(
            x=x,
            y=y,
            includeUserAgentShadowDOM=include_user_agent_shadow_dom,
            ignorePointerEventsNone=ignore_pointer_events_none
        )


class GetOuterHTMLInput(InputModel):

    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectId: Runtime.RemoteObjectId | None = None
    includeShadowDOM: bool | None = None  # experimental


class GetOuterHTMLOutput(OutputModel):

    outerHTML: str


class GetOuterHTML(CDPMethod[GetOuterHTMLOutput]):
    """ Returns node's HTML markup. """

    INPUT_VALIDATOR = GetOuterHTMLInput
    OUTPUT_VALIDATOR = GetOuterHTMLOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_id: Runtime.RemoteObjectId | None = None,
        include_shadow_dom: bool | None = None
    ):
        super().__init__(
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectId=object_id,
            includeShadowDOM=include_shadow_dom
        )


class GetRelayoutBoundaryInput(InputModel):

    nodeId: DOM.NodeId


class GetRelayoutBoundaryOutput(OutputModel):

    nodeId: DOM.NodeId


class GetRelayoutBoundary(CDPMethod[GetRelayoutBoundaryOutput]):  # experimental
    """ Returns the id of the nearest ancestor that is a relayout boundary. """

    INPUT_VALIDATOR = GetRelayoutBoundaryInput
    OUTPUT_VALIDATOR = GetRelayoutBoundaryOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId
    ):
        super().__init__(
            nodeId=node_id
        )


class GetSearchResultsInput(InputModel):

    searchId: str
    fromIndex: int
    toIndex: int


class GetSearchResultsOutput(OutputModel):

    nodeIds: list[DOM.NodeId]


class GetSearchResults(CDPMethod[GetSearchResultsOutput]):  # experimental
    """ Returns search results from given `fromIndex` to given `toIndex` from the search with the given
    identifier. """

    INPUT_VALIDATOR = GetSearchResultsInput
    OUTPUT_VALIDATOR = GetSearchResultsOutput

    def __init__(
        self,
        *,
        search_id: str,
        from_index: int,
        to_index: int
    ):
        super().__init__(
            searchId=search_id,
            fromIndex=from_index,
            toIndex=to_index
        )


class HideHighlight(CDPMethod[None]):
    """ Hides any highlight. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class HighlightNode(CDPMethod[None]):
    """ Highlights DOM node. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class HighlightRect(CDPMethod[None]):
    """ Highlights given rectangle. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class MarkUndoableState(CDPMethod[None]):  # experimental
    """ Marks last undoable state. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class MoveToInput(InputModel):

    nodeId: DOM.NodeId
    targetNodeId: DOM.NodeId
    insertBeforeNodeId: DOM.NodeId | None = None


class MoveToOutput(OutputModel):

    nodeId: DOM.NodeId


class MoveTo(CDPMethod[MoveToOutput]):
    """ Moves node into the new container, places it before the given anchor. """

    INPUT_VALIDATOR = MoveToInput
    OUTPUT_VALIDATOR = MoveToOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        target_node_id: DOM.NodeId,
        insert_before_node_id: DOM.NodeId | None = None
    ):
        super().__init__(
            nodeId=node_id,
            targetNodeId=target_node_id,
            insertBeforeNodeId=insert_before_node_id
        )


class PerformSearchInput(InputModel):

    query: str
    includeUserAgentShadowDOM: bool | None = None


class PerformSearchOutput(OutputModel):

    searchId: str
    resultCount: int


class PerformSearch(CDPMethod[PerformSearchOutput]):  # experimental
    """ Searches for a given string in the DOM tree. Use `getSearchResults` to access search results or
    `cancelSearch` to end this search session. """

    INPUT_VALIDATOR = PerformSearchInput
    OUTPUT_VALIDATOR = PerformSearchOutput

    def __init__(
        self,
        *,
        query: str,
        include_user_agent_shadow_dom: bool | None = None
    ):
        super().__init__(
            query=query,
            includeUserAgentShadowDOM=include_user_agent_shadow_dom
        )


class PushNodeByPathToFrontendInput(InputModel):

    path: str


class PushNodeByPathToFrontendOutput(OutputModel):

    nodeId: DOM.NodeId


class PushNodeByPathToFrontend(CDPMethod[PushNodeByPathToFrontendOutput]):  # experimental
    """ Requests that the node is sent to the caller given its path. // FIXME, use XPath """

    INPUT_VALIDATOR = PushNodeByPathToFrontendInput
    OUTPUT_VALIDATOR = PushNodeByPathToFrontendOutput

    def __init__(
        self,
        *,
        path: str
    ):
        super().__init__(
            path=path
        )


class PushNodesByBackendIdsToFrontendInput(InputModel):

    backendNodeIds: list[DOM.BackendNodeId]


class PushNodesByBackendIdsToFrontendOutput(OutputModel):

    nodeIds: list[DOM.NodeId]


class PushNodesByBackendIdsToFrontend(CDPMethod[PushNodesByBackendIdsToFrontendOutput]):  # experimental
    """ Requests that a batch of nodes is sent to the caller given their backend node ids. """

    INPUT_VALIDATOR = PushNodesByBackendIdsToFrontendInput
    OUTPUT_VALIDATOR = PushNodesByBackendIdsToFrontendOutput

    def __init__(
        self,
        *,
        backend_node_ids: list[DOM.BackendNodeId]
    ):
        super().__init__(
            backendNodeIds=backend_node_ids
        )


class QuerySelectorInput(InputModel):

    nodeId: DOM.NodeId
    selector: str


class QuerySelectorOutput(OutputModel):

    nodeId: DOM.NodeId


class QuerySelector(CDPMethod[QuerySelectorOutput]):
    """ Executes `querySelector` on a given node. """

    INPUT_VALIDATOR = QuerySelectorInput
    OUTPUT_VALIDATOR = QuerySelectorOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        selector: str
    ):
        super().__init__(
            nodeId=node_id,
            selector=selector
        )


class QuerySelectorAllInput(InputModel):

    nodeId: DOM.NodeId
    selector: str


class QuerySelectorAllOutput(OutputModel):

    nodeIds: list[DOM.NodeId]


class QuerySelectorAll(CDPMethod[QuerySelectorAllOutput]):
    """ Executes `querySelectorAll` on a given node. """

    INPUT_VALIDATOR = QuerySelectorAllInput
    OUTPUT_VALIDATOR = QuerySelectorAllOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        selector: str
    ):
        super().__init__(
            nodeId=node_id,
            selector=selector
        )


class GetTopLayerElementsOutput(OutputModel):

    nodeIds: list[DOM.NodeId]


class GetTopLayerElements(CDPMethod[GetTopLayerElementsOutput]):  # experimental
    """ Returns NodeIds of current top layer elements.
    Top layer is rendered closest to the user within a viewport, therefore its elements always
    appear on top of all other content. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetTopLayerElementsOutput


class GetElementByRelationInput(InputModel):

    nodeId: DOM.NodeId
    relation: Literal['PopoverTarget', 'InterestTarget', 'CommandFor']


class GetElementByRelationOutput(OutputModel):

    nodeId: DOM.NodeId


class GetElementByRelation(CDPMethod[GetElementByRelationOutput]):  # experimental
    """ Returns the NodeId of the matched element according to certain relations. """

    INPUT_VALIDATOR = GetElementByRelationInput
    OUTPUT_VALIDATOR = GetElementByRelationOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        relation: Literal['PopoverTarget', 'InterestTarget', 'CommandFor']
    ):
        super().__init__(
            nodeId=node_id,
            relation=relation
        )


class Redo(CDPMethod[None]):  # experimental
    """ Re-does the last undone action. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class RemoveAttributeInput(InputModel):

    nodeId: DOM.NodeId
    name: str


class RemoveAttribute(CDPMethod[None]):
    """ Removes attribute with given name from an element with given id. """

    INPUT_VALIDATOR = RemoveAttributeInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        name: str
    ):
        super().__init__(
            nodeId=node_id,
            name=name
        )


class RemoveNodeInput(InputModel):

    nodeId: DOM.NodeId


class RemoveNode(CDPMethod[None]):
    """ Removes node with given id. """

    INPUT_VALIDATOR = RemoveNodeInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId
    ):
        super().__init__(
            nodeId=node_id
        )


class RequestChildNodesInput(InputModel):

    nodeId: DOM.NodeId
    depth: int | None = None
    pierce: bool | None = None


class RequestChildNodes(CDPMethod[None]):
    """ Requests that children of the node with given id are returned to the caller in form of
    `setChildNodes` events where not only immediate children are retrieved, but all children down to
    the specified depth. """

    INPUT_VALIDATOR = RequestChildNodesInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        depth: int | None = None,
        pierce: bool | None = None
    ):
        super().__init__(
            nodeId=node_id,
            depth=depth,
            pierce=pierce
        )


class RequestNodeInput(InputModel):

    objectId: Runtime.RemoteObjectId


class RequestNodeOutput(OutputModel):

    nodeId: DOM.NodeId


class RequestNode(CDPMethod[RequestNodeOutput]):
    """ Requests that the node is sent to the caller given the JavaScript node object reference. All
    nodes that form the path from the node to the root are also sent to the client as a series of
    `setChildNodes` notifications. """

    INPUT_VALIDATOR = RequestNodeInput
    OUTPUT_VALIDATOR = RequestNodeOutput

    def __init__(
        self,
        *,
        object_id: Runtime.RemoteObjectId
    ):
        super().__init__(
            objectId=object_id
        )


class ResolveNodeInput(InputModel):

    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectGroup: str | None = None
    executionContextId: Runtime.ExecutionContextId | None = None


class ResolveNodeOutput(OutputModel):

    object: Runtime.RemoteObject


class ResolveNode(CDPMethod[ResolveNodeOutput]):
    """ Resolves the JavaScript node object for a given NodeId or BackendNodeId. """

    INPUT_VALIDATOR = ResolveNodeInput
    OUTPUT_VALIDATOR = ResolveNodeOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_group: str | None = None,
        execution_context_id: Runtime.ExecutionContextId | None = None
    ):
        super().__init__(
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectGroup=object_group,
            executionContextId=execution_context_id
        )


class SetAttributeValueInput(InputModel):

    nodeId: DOM.NodeId
    name: str
    value: str


class SetAttributeValue(CDPMethod[None]):
    """ Sets attribute for an element with given id. """

    INPUT_VALIDATOR = SetAttributeValueInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        name: str,
        value: str
    ):
        super().__init__(
            nodeId=node_id,
            name=name,
            value=value
        )


class SetAttributesAsTextInput(InputModel):

    nodeId: DOM.NodeId
    text: str
    name: str | None = None


class SetAttributesAsText(CDPMethod[None]):
    """ Sets attributes on element with given id. This method is useful when user edits some existing
    attribute value and types in several attribute name/value pairs. """

    INPUT_VALIDATOR = SetAttributesAsTextInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        text: str,
        name: str | None = None
    ):
        super().__init__(
            nodeId=node_id,
            text=text,
            name=name
        )


class SetFileInputFilesInput(InputModel):

    files: list[str]
    nodeId: DOM.NodeId | None = None
    backendNodeId: DOM.BackendNodeId | None = None
    objectId: Runtime.RemoteObjectId | None = None


class SetFileInputFiles(CDPMethod[None]):
    """ Sets files for the given file input element. """

    INPUT_VALIDATOR = SetFileInputFilesInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        files: list[str],
        node_id: DOM.NodeId | None = None,
        backend_node_id: DOM.BackendNodeId | None = None,
        object_id: Runtime.RemoteObjectId | None = None
    ):
        super().__init__(
            files=files,
            nodeId=node_id,
            backendNodeId=backend_node_id,
            objectId=object_id
        )


class SetNodeStackTracesEnabledInput(InputModel):

    enable: bool


class SetNodeStackTracesEnabled(CDPMethod[None]):  # experimental
    """ Sets if stack traces should be captured for Nodes. See `Node.getNodeStackTraces`. Default is disabled. """

    INPUT_VALIDATOR = SetNodeStackTracesEnabledInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        enable: bool
    ):
        super().__init__(
            enable=enable
        )


class GetNodeStackTracesInput(InputModel):

    nodeId: DOM.NodeId


class GetNodeStackTracesOutput(OutputModel):

    creation: Runtime.StackTrace | None = None


class GetNodeStackTraces(CDPMethod[GetNodeStackTracesOutput]):  # experimental
    """ Gets stack traces associated with a Node. As of now, only provides stack trace for Node creation. """

    INPUT_VALIDATOR = GetNodeStackTracesInput
    OUTPUT_VALIDATOR = GetNodeStackTracesOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId
    ):
        super().__init__(
            nodeId=node_id
        )


class GetFileInfoInput(InputModel):

    objectId: Runtime.RemoteObjectId


class GetFileInfoOutput(OutputModel):

    path: str


class GetFileInfo(CDPMethod[GetFileInfoOutput]):  # experimental
    """ Returns file information for the given
    File wrapper. """

    INPUT_VALIDATOR = GetFileInfoInput
    OUTPUT_VALIDATOR = GetFileInfoOutput

    def __init__(
        self,
        *,
        object_id: Runtime.RemoteObjectId
    ):
        super().__init__(
            objectId=object_id
        )


class GetDetachedDomNodesOutput(OutputModel):

    detachedNodes: list[DOM.DetachedElementInfo]


class GetDetachedDomNodes(CDPMethod[GetDetachedDomNodesOutput]):  # experimental
    """ Returns list of detached nodes """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetDetachedDomNodesOutput


class SetInspectedNodeInput(InputModel):

    nodeId: DOM.NodeId


class SetInspectedNode(CDPMethod[None]):  # experimental
    """ Enables console to refer to the node with given id via $x (see Command Line API for more details
    $x functions). """

    INPUT_VALIDATOR = SetInspectedNodeInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId
    ):
        super().__init__(
            nodeId=node_id
        )


class SetNodeNameInput(InputModel):

    nodeId: DOM.NodeId
    name: str


class SetNodeNameOutput(OutputModel):

    nodeId: DOM.NodeId


class SetNodeName(CDPMethod[SetNodeNameOutput]):
    """ Sets node name for a node with given id. """

    INPUT_VALIDATOR = SetNodeNameInput
    OUTPUT_VALIDATOR = SetNodeNameOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        name: str
    ):
        super().__init__(
            nodeId=node_id,
            name=name
        )


class SetNodeValueInput(InputModel):

    nodeId: DOM.NodeId
    value: str


class SetNodeValue(CDPMethod[None]):
    """ Sets node value for a node with given id. """

    INPUT_VALIDATOR = SetNodeValueInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        value: str
    ):
        super().__init__(
            nodeId=node_id,
            value=value
        )


class SetOuterHTMLInput(InputModel):

    nodeId: DOM.NodeId
    outerHTML: str


class SetOuterHTML(CDPMethod[None]):
    """ Sets node HTML markup, returns new node id. """

    INPUT_VALIDATOR = SetOuterHTMLInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        outer_html: str
    ):
        super().__init__(
            nodeId=node_id,
            outerHTML=outer_html
        )


class Undo(CDPMethod[None]):  # experimental
    """ Undoes the last performed action. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class GetFrameOwnerInput(InputModel):

    frameId: Page.FrameId


class GetFrameOwnerOutput(OutputModel):

    backendNodeId: DOM.BackendNodeId
    nodeId: DOM.NodeId | None = None


class GetFrameOwner(CDPMethod[GetFrameOwnerOutput]):  # experimental
    """ Returns iframe node that owns iframe with the given domain. """

    INPUT_VALIDATOR = GetFrameOwnerInput
    OUTPUT_VALIDATOR = GetFrameOwnerOutput

    def __init__(
        self,
        *,
        frame_id: Page.FrameId
    ):
        super().__init__(
            frameId=frame_id
        )


class GetContainerForNodeInput(InputModel):

    nodeId: DOM.NodeId
    containerName: str | None = None
    physicalAxes: DOM.PhysicalAxes | None = None
    logicalAxes: DOM.LogicalAxes | None = None
    queriesScrollState: bool | None = None
    queriesAnchored: bool | None = None


class GetContainerForNodeOutput(OutputModel):

    nodeId: DOM.NodeId | None = None


class GetContainerForNode(CDPMethod[GetContainerForNodeOutput]):  # experimental
    """ Returns the query container of the given node based on container query
    conditions: containerName, physical and logical axes, and whether it queries
    scroll-state or anchored elements. If no axes are provided and
    queriesScrollState is false, the style container is returned, which is the
    direct parent or the closest element with a matching container-name. """

    INPUT_VALIDATOR = GetContainerForNodeInput
    OUTPUT_VALIDATOR = GetContainerForNodeOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        container_name: str | None = None,
        physical_axes: DOM.PhysicalAxes | None = None,
        logical_axes: DOM.LogicalAxes | None = None,
        queries_scroll_state: bool | None = None,
        queries_anchored: bool | None = None
    ):
        super().__init__(
            nodeId=node_id,
            containerName=container_name,
            physicalAxes=physical_axes,
            logicalAxes=logical_axes,
            queriesScrollState=queries_scroll_state,
            queriesAnchored=queries_anchored
        )


class GetQueryingDescendantsForContainerInput(InputModel):

    nodeId: DOM.NodeId


class GetQueryingDescendantsForContainerOutput(OutputModel):

    nodeIds: list[DOM.NodeId]


class GetQueryingDescendantsForContainer(CDPMethod[GetQueryingDescendantsForContainerOutput]):  # experimental
    """ Returns the descendants of a container query container that have
    container queries against this container. """

    INPUT_VALIDATOR = GetQueryingDescendantsForContainerInput
    OUTPUT_VALIDATOR = GetQueryingDescendantsForContainerOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId
    ):
        super().__init__(
            nodeId=node_id
        )


class GetAnchorElementInput(InputModel):

    nodeId: DOM.NodeId
    anchorSpecifier: str | None = None


class GetAnchorElementOutput(OutputModel):

    nodeId: DOM.NodeId


class GetAnchorElement(CDPMethod[GetAnchorElementOutput]):  # experimental
    """ Returns the target anchor element of the given anchor query according to
    https://www.w3.org/TR/css-anchor-position-1/#target. """

    INPUT_VALIDATOR = GetAnchorElementInput
    OUTPUT_VALIDATOR = GetAnchorElementOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        anchor_specifier: str | None = None
    ):
        super().__init__(
            nodeId=node_id,
            anchorSpecifier=anchor_specifier
        )


class ForceShowPopoverInput(InputModel):

    nodeId: DOM.NodeId
    enable: bool


class ForceShowPopoverOutput(OutputModel):

    nodeIds: list[DOM.NodeId]


class ForceShowPopover(CDPMethod[ForceShowPopoverOutput]):  # experimental
    """ When enabling, this API force-opens the popover identified by nodeId
    and keeps it open until disabled. """

    INPUT_VALIDATOR = ForceShowPopoverInput
    OUTPUT_VALIDATOR = ForceShowPopoverOutput

    def __init__(
        self,
        *,
        node_id: DOM.NodeId,
        enable: bool
    ):
        super().__init__(
            nodeId=node_id,
            enable=enable
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/Animation/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Runtime,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class Disable(CDPMethod[None]):
    """ Disables animation domain notifications. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables animation domain notifications. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class GetCurrentTimeInput(InputModel):

    id: str


class GetCurrentTimeOutput(OutputModel):

    currentTime: float


class GetCurrentTime(CDPMethod[GetCurrentTimeOutput]):
    """ Returns the current time of the an animation. """

    INPUT_VALIDATOR = GetCurrentTimeInput
    OUTPUT_VALIDATOR = GetCurrentTimeOutput

    def __init__(
        self,
        *,
        id_: str
    ):
        super().__init__(
            id=id_
        )


class GetPlaybackRateOutput(OutputModel):

    playbackRate: float


class GetPlaybackRate(CDPMethod[GetPlaybackRateOutput]):
    """ Gets the playback rate of the document timeline. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetPlaybackRateOutput


class ReleaseAnimationsInput(InputModel):

    animations: list[str]


class ReleaseAnimations(CDPMethod[None]):
    """ Releases a set of animations to no longer be manipulated. """

    INPUT_VALIDATOR = ReleaseAnimationsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        animations: list[str]
    ):
        super().__init__(
            animations=animations
        )


class ResolveAnimationInput(InputModel):

    animationId: str


class ResolveAnimationOutput(OutputModel):

    remoteObject: Runtime.RemoteObject


class ResolveAnimation(CDPMethod[ResolveAnimationOutput]):
    """ Gets the remote object of the Animation. """

    INPUT_VALIDATOR = ResolveAnimationInput
    OUTPUT_VALIDATOR = ResolveAnimationOutput

    def __init__(
        self,
        *,
        animation_id: str
    ):
        super().__init__(
            animationId=animation_id
        )


class SeekAnimationsInput(InputModel):

    animations: list[str]
    currentTime: float


class SeekAnimations(CDPMethod[None]):
    """ Seek a set of animations to a particular time within each animation. """

    INPUT_VALIDATOR = SeekAnimationsInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        animations: list[str],
        current_time: float
    ):
        super().__init__(
            animations=animations,
            currentTime=current_time
        )


class SetPausedInput(InputModel):

    animations: list[str]
    paused: bool


class SetPaused(CDPMethod[None]):
    """ Sets the paused state of a set of animations. """

    INPUT_VALIDATOR = SetPausedInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        animations: list[str],
        paused: bool
    ):
        super().__init__(
            animations=animations,
            paused=paused
        )


class SetPlaybackRateInput(InputModel):

    playbackRate: float


class SetPlaybackRate(CDPMethod[None]):
    """ Sets the playback rate of the document timeline. """

    INPUT_VALIDATOR = SetPlaybackRateInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        playback_rate: float
    ):
        super().__init__(
            playbackRate=playback_rate
        )


class SetTimingInput(InputModel):

    animationId: str
    duration: float
    delay: float


class SetTiming(CDPMethod[None]):
    """ Sets the timing of an animation node. """

    INPUT_VALIDATOR = SetTimingInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        animation_id: str,
        duration: float,
        delay: float
    ):
        super().__init__(
            animationId=animation_id,
            duration=duration,
            delay=delay
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.


https://chromedevtools.github.io/devtools-protocol/tot/IndexedDB/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    IndexedDB,
    Storage,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class ClearObjectStoreInput(InputModel):

    securityOrigin: str | None = None
    storageKey: str | None = None
    storageBucket: Storage.StorageBucket | None = None
    databaseName: str
    objectStoreName: str


class ClearObjectStore(CDPMethod[None]):
    """ Clears all entries from an object store. """

    INPUT_VALIDATOR = ClearObjectStoreInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        security_origin: str | None = None,
        storage_key: str | None = None,
        storage_bucket: Storage.StorageBucket | None = None,
        database_name: str,
        object_store_name: str
    ):
        super().__init__(
            securityOrigin=security_origin,
            storageKey=storage_key,
            storageBucket=storage_bucket,
            databaseName=database_name,
            objectStoreName=object_store_name
        )


class DeleteDatabaseInput(InputModel):

    securityOrigin: str | None = None
    storageKey: str | None = None
    storageBucket: Storage.StorageBucket | None = None
    databaseName: str


class DeleteDatabase(CDPMethod[None]):
    """ Deletes a database. """

    INPUT_VALIDATOR = DeleteDatabaseInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        security_origin: str | None = None,
        storage_key: str | None = None,
        storage_bucket: Storage.StorageBucket | None = None,
        database_name: str
    ):
        super().__init__(
            securityOrigin=security_origin,
            storageKey=storage_key,
            storageBucket=storage_bucket,
            databaseName=database_name
        )


class DeleteObjectStoreEntriesInput(InputModel):

    securityOrigin: str | None = None
    storageKey: str | None = None
    storageBucket: Storage.StorageBucket | None = None
    databaseName: str
    objectStoreName: str
    keyRange: IndexedDB.KeyRange


class DeleteObjectStoreEntries(CDPMethod[None]):
    """ Delete a range of entries from an object store """

    INPUT_VALIDATOR = DeleteObjectStoreEntriesInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        security_origin: str | None = None,
        storage_key: str | None = None,
        storage_bucket: Storage.StorageBucket | None = None,
        database_name: str,
        object_store_name: str,
        key_range: IndexedDB.KeyRange
    ):
        super().__init__(
            securityOrigin=security_origin,
            storageKey=storage_key,
            storageBucket=storage_bucket,
            databaseName=database_name,
            objectStoreName=object_store_name,
            keyRange=key_range
        )


class Disable(CDPMethod[None]):
    """ Disables events from backend. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables events from backend. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class RequestDataInput(InputModel):

    securityOrigin: str | None = None
    storageKey: str | None = None
    storageBucket: Storage.StorageBucket | None = None
    databaseName: str
    objectStoreName: str
    indexName: str | None = None
    skipCount: int
    pageSize: int
    keyRange: IndexedDB.KeyRange | None = None


class RequestDataOutput(OutputModel):

    objectStoreDataEntries: list[IndexedDB.DataEntry]
    hasMore: bool


class RequestData(CDPMethod[RequestDataOutput]):
    """ Requests data from object store or index. """

    INPUT_VALIDATOR = RequestDataInput
    OUTPUT_VALIDATOR = RequestDataOutput

    def __init__(
        self,
        *,
        security_origin: str | None = None,
        storage_key: str | None = None,
        storage_bucket: Storage.StorageBucket | None = None,
        database_name: str,
        object_store_name: str,
        index_name: str | None = None,
        skip_count: int,
        page_size: int,
        key_range: IndexedDB.KeyRange | None = None
    ):
        super().__init__(
            securityOrigin=security_origin,
            storageKey=storage_key,
            storageBucket=storage_bucket,
            databaseName=database_name,
            objectStoreName=object_store_name,
            indexName=index_name,
            skipCount=skip_count,
            pageSize=page_size,
            keyRange=key_range
        )


class GetMetadataInput(InputModel):

    securityOrigin: str | None = None
    storageKey: str | None = None
    storageBucket: Storage.StorageBucket | None = None
    databaseName: str
    objectStoreName: str


class GetMetadataOutput(OutputModel):

    entriesCount: float
    keyGeneratorValue: float


class GetMetadata(CDPMethod[GetMetadataOutput]):
    """ Gets metadata of an object store. """

    INPUT_VALIDATOR = GetMetadataInput
    OUTPUT_VALIDATOR = GetMetadataOutput

    def __init__(
        self,
        *,
        security_origin: str | None = None,
        storage_key: str | None = None,
        storage_bucket: Storage.StorageBucket | None = None,
        database_name: str,
        object_store_name: str
    ):
        super().__init__(
            securityOrigin=security_origin,
            storageKey=storage_key,
            storageBucket=storage_bucket,
            databaseName=database_name,
            objectStoreName=object_store_name
        )


class RequestDatabaseInput(InputModel):

    securityOrigin: str | None = None
    storageKey: str | None = None
    storageBucket: Storage.StorageBucket | None = None
    databaseName: str


class RequestDatabaseOutput(OutputModel):

    databaseWithObjectStores: IndexedDB.DatabaseWithObjectStores


class RequestDatabase(CDPMethod[RequestDatabaseOutput]):
    """ Requests database with given name in given frame. """

    INPUT_VALIDATOR = RequestDatabaseInput
    OUTPUT_VALIDATOR = RequestDatabaseOutput

    def __init__(
        self,
        *,
        security_origin: str | None = None,
        storage_key: str | None = None,
        storage_bucket: Storage.StorageBucket | None = None,
        database_name: str
    ):
        super().__init__(
            securityOrigin=security_origin,
            storageKey=storage_key,
            storageBucket=storage_bucket,
            databaseName=database_name
        )


class RequestDatabaseNamesInput(InputModel):

    securityOrigin: str | None = None
    storageKey: str | None = None
    storageBucket: Storage.StorageBucket | None = None


class RequestDatabaseNamesOutput(OutputModel):

    databaseNames: list[str]


class RequestDatabaseNames(CDPMethod[RequestDatabaseNamesOutput]):
    """ Requests database names for given security origin. """

    INPUT_VALIDATOR = RequestDatabaseNamesInput
    OUTPUT_VALIDATOR = RequestDatabaseNamesOutput

    def __init__(
        self,
        *,
        security_origin: str | None = None,
        storage_key: str | None = None,
        storage_bucket: Storage.StorageBucket | None = None
    ):
        super().__init__(
            securityOrigin=security_origin,
            storageKey=storage_key,
            storageBucket=storage_bucket
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

A domain for letting clients substitute browser's network layer with client code.
https://chromedevtools.github.io/devtools-protocol/tot/Fetch/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    IO,
    Fetch,
    Network,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class Disable(CDPMethod[None]):
    """ Disables the fetch domain. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class EnableInput(InputModel):

    patterns: list[Fetch.RequestPattern] | None = None
    handleAuthRequests: bool | None = None


class Enable(CDPMethod[None]):
    """ Enables issuing of requestPaused events. A request will be paused until client
    calls one of failRequest, fulfillRequest or continueRequest/continueWithAuth. """

    INPUT_VALIDATOR = EnableInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        patterns: list[Fetch.RequestPattern] | None = None,
        handle_auth_requests: bool | None = None
    ):
        super().__init__(
            patterns=patterns,
            handleAuthRequests=handle_auth_requests
        )


class FailRequestInput(InputModel):

    requestId: Fetch.RequestId
    errorReason: Network.ErrorReason


class FailRequest(CDPMethod[None]):
    """ Causes the request to fail with specified reason. """

    INPUT_VALIDATOR = FailRequestInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        request_id: Fetch.RequestId,
        error_reason: Network.ErrorReason
    ):
        super().__init__(
            requestId=request_id,
            errorReason=error_reason
        )


class FulfillRequestInput(InputModel):

    requestId: Fetch.RequestId
    responseCode: int
    responseHeaders: list[Fetch.HeaderEntry] | None = None
    binaryResponseHeaders: str | None = None
    body: str | None = None
    responsePhrase: str | None = None


class FulfillRequest(CDPMethod[None]):
    """ Provides response to the request. """

    INPUT_VALIDATOR = FulfillRequestInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        request_id: Fetch.RequestId,
        response_code: int,
        response_headers: list[Fetch.HeaderEntry] | None = None,
        binary_response_headers: str | None = None,
        body: str | None = None,
        response_phrase: str | None = None
    ):
        super().__init__(
            requestId=request_id,
            responseCode=response_code,
            responseHeaders=response_headers,
            binaryResponseHeaders=binary_response_headers,
            body=body,
            responsePhrase=response_phrase
        )


class ContinueRequestInput(InputModel):

    requestId: Fetch.RequestId
    url: str | None = None
    method: str | None = None
    postData: str | None = None
    headers: list[Fetch.HeaderEntry] | None = None
    interceptResponse: bool | None = None  # experimental


class ContinueRequest(CDPMethod[None]):
    """ Continues the request, optionally modifying some of its parameters. """

    INPUT_VALIDATOR = ContinueRequestInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        request_id: Fetch.RequestId,
        url: str | None = None,
        method: str | None = None,
        post_data: str | None = None,
        headers: list[Fetch.HeaderEntry] | None = None,
        intercept_response: bool | None = None
    ):
        super().__init__(
            requestId=request_id,
            url=url,
            method=method,
            postData=post_data,
            headers=headers,
            interceptResponse=intercept_response
        )


class ContinueWithAuthInput(InputModel):

    requestId: Fetch.RequestId
    authChallengeResponse: Fetch.AuthChallengeResponse


class ContinueWithAuth(CDPMethod[None]):
    """ Continues a request supplying authChallengeResponse following authRequired event. """

    INPUT_VALIDATOR = ContinueWithAuthInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        request_id: Fetch.RequestId,
        auth_challenge_response: Fetch.AuthChallengeResponse
    ):
        super().__init__(
            requestId=request_id,
            authChallengeResponse=auth_challenge_response
        )


class ContinueResponseInput(InputModel):

    requestId: Fetch.RequestId
    responseCode: int | None = None
    responsePhrase: str | None = None
    responseHeaders: list[Fetch.HeaderEntry] | None = None
    binaryResponseHeaders: str | None = None


class ContinueResponse(CDPMethod[None]):  # experimental
    """ Continues loading of the paused response, optionally modifying the
    response headers. If either responseCode or headers are modified, all of them
    must be present. """

    INPUT_VALIDATOR = ContinueResponseInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        request_id: Fetch.RequestId,
        response_code: int | None = None,
        response_phrase: str | None = None,
        response_headers: list[Fetch.HeaderEntry] | None = None,
        binary_response_headers: str | None = None
    ):
        super().__init__(
            requestId=request_id,
            responseCode=response_code,
            responsePhrase=response_phrase,
            responseHeaders=response_headers,
            binaryResponseHeaders=binary_response_headers
        )


class GetResponseBodyInput(InputModel):

    requestId: Fetch.RequestId


class GetResponseBodyOutput(OutputModel):

    body: str
    base64Encoded: bool


class GetResponseBody(CDPMethod[GetResponseBodyOutput]):
    """ Causes the body of the response to be received from the server and
    returned as a single string. May only be issued for a request that
    is paused in the Response stage and is mutually exclusive with
    takeResponseBodyForInterceptionAsStream. Calling other methods that
    affect the request or disabling fetch domain before body is received
    results in an undefined behavior.
    Note that the response body is not available for redirects. Requests
    paused in the _redirect received_ state may be differentiated by
    `responseCode` and presence of `location` response header, see
    comments to `requestPaused` for details. """

    INPUT_VALIDATOR = GetResponseBodyInput
    OUTPUT_VALIDATOR = GetResponseBodyOutput

    def __init__(
        self,
        *,
        request_id: Fetch.RequestId
    ):
        super().__init__(
            requestId=request_id
        )


class TakeResponseBodyAsStreamInput(InputModel):

    requestId: Fetch.RequestId


class TakeResponseBodyAsStreamOutput(OutputModel):

    stream: IO.StreamHandle


class TakeResponseBodyAsStream(CDPMethod[TakeResponseBodyAsStreamOutput]):
    """ Returns a handle to the stream representing the response body.
    The request must be paused in the HeadersReceived stage.
    Note that after this command the request can't be continued
    as is -- client either needs to cancel it or to provide the
    response body.
    The stream only supports sequential read, IO.read will fail if the position
    is specified.
    This method is mutually exclusive with getResponseBody.
    Calling other methods that affect the request or disabling fetch
    domain before body is received results in an undefined behavior. """

    INPUT_VALIDATOR = TakeResponseBodyAsStreamInput
    OUTPUT_VALIDATOR = TakeResponseBodyAsStreamOutput

    def __init__(
        self,
        *,
        request_id: Fetch.RequestId
    ):
        super().__init__(
            requestId=request_id
        )



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain is deprecated - use Runtime or Log instead.
https://chromedevtools.github.io/devtools-protocol/tot/Console/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol.base import CDPMethod


class ClearMessages(CDPMethod[None]):
    """ Does nothing. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Disable(CDPMethod[None]):
    """ Disables console domain, prevents further console messages from being reported to the client. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables console domain, sends the messages collected so far to the client by means of the
    `messageAdded` notification. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None



from .events import BreakpointResolved, Paused, Resumed, ScriptFailedToParse, ScriptParsed
from .methods import (
    ContinueToLocation,
    Disable,
    DisassembleWasmModule,
    Enable,
    EvaluateOnCallFrame,
    GetPossibleBreakpoints,
    GetScriptSource,
    GetStackTrace,
    GetWasmBytecode,
    NextWasmDisassemblyChunk,
    Pause,
    PauseOnAsyncCall,
    RemoveBreakpoint,
    RestartFrame,
    Resume,
    SearchInContent,
    SetAsyncCallStackDepth,
    SetBlackboxedRanges,
    SetBlackboxExecutionContexts,
    SetBlackboxPatterns,
    SetBreakpoint,
    SetBreakpointByUrl,
    SetBreakpointOnFunctionCall,
    SetBreakpointsActive,
    SetInstrumentationBreakpoint,
    SetPauseOnExceptions,
    SetReturnValue,
    SetScriptSource,
    SetSkipAllPauses,
    SetVariableValue,
    StepInto,
    StepOut,
    StepOver,
)
from .types import (
    BreakLocation,
    BreakpointId,
    CallFrame,
    CallFrameId,
    DebugSymbols,
    Location,
    LocationRange,
    ResolvedBreakpoint,
    Scope,
    ScriptLanguage,
    ScriptPosition,
    SearchMatch,
    WasmDisassemblyChunk,
)

__all__ = [
    BreakpointId,
    CallFrameId,
    Location,
    ScriptPosition,
    LocationRange,
    CallFrame,
    Scope,
    SearchMatch,
    BreakLocation,
    WasmDisassemblyChunk,
    ScriptLanguage,
    DebugSymbols,
    ResolvedBreakpoint,
    BreakpointResolved,
    Paused,
    Resumed,
    ScriptFailedToParse,
    ScriptParsed,
    ContinueToLocation,
    Disable,
    Enable,
    EvaluateOnCallFrame,
    GetPossibleBreakpoints,
    GetScriptSource,
    DisassembleWasmModule,
    NextWasmDisassemblyChunk,
    GetWasmBytecode,
    GetStackTrace,
    Pause,
    PauseOnAsyncCall,
    RemoveBreakpoint,
    RestartFrame,
    Resume,
    SearchInContent,
    SetAsyncCallStackDepth,
    SetBlackboxExecutionContexts,
    SetBlackboxPatterns,
    SetBlackboxedRanges,
    SetBreakpoint,
    SetInstrumentationBreakpoint,
    SetBreakpointByUrl,
    SetBreakpointOnFunctionCall,
    SetBreakpointsActive,
    SetPauseOnExceptions,
    SetReturnValue,
    SetScriptSource,
    SetSkipAllPauses,
    SetVariableValue,
    StepInto,
    StepOut,
    StepOver,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

DOM debugging allows setting breakpoints on particular DOM operations and events. JavaScript
execution will stop on these operations as if there was a regular breakpoint set.
https://chromedevtools.github.io/devtools-protocol/tot/DOMDebugger/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

A domain for interacting with Cast, Presentation API, and Remote Playback API
functionalities.
https://chromedevtools.github.io/devtools-protocol/tot/Cast/

***************************************************
                    Types
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    CastSink,
)

Sink = CastSink



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain allows interacting with the FedCM dialog.
https://chromedevtools.github.io/devtools-protocol/tot/FedCm/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    FedCm,
)
from cdpkit.protocol.base import CDPEvent


class DialogShown(CDPEvent):

    dialogId: str
    dialogType: FedCm.DialogType
    accounts: list[FedCm.Account]
    title: str
    subtitle: str | None = None


class DialogClosed(CDPEvent):
    """ Triggered when a dialog is closed, either by user action, JS abort,
    or a command below. """

    dialogId: str



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

EventBreakpoints permits setting JavaScript breakpoints on operations and events
occurring in native code invoked from JavaScript. Once breakpoint is hit, it is
reported through Debugger domain, similarly to regular breakpoints being hit.
https://chromedevtools.github.io/devtools-protocol/tot/EventBreakpoints/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol.base import CDPMethod, InputModel


class SetInstrumentationBreakpointInput(InputModel):

    eventName: str


class SetInstrumentationBreakpoint(CDPMethod[None]):
    """ Sets breakpoint on particular native event. """

    INPUT_VALIDATOR = SetInstrumentationBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        event_name: str
    ):
        super().__init__(
            eventName=event_name
        )


class RemoveInstrumentationBreakpointInput(InputModel):

    eventName: str


class RemoveInstrumentationBreakpoint(CDPMethod[None]):
    """ Removes breakpoint on particular native event. """

    INPUT_VALIDATOR = RemoveInstrumentationBreakpointInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        event_name: str
    ):
        super().__init__(
            eventName=event_name
        )


class Disable(CDPMethod[None]):
    """ Removes all breakpoints """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

The Browser domain defines methods and events for browser managing.
https://chromedevtools.github.io/devtools-protocol/tot/Browser/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from typing import Literal

from cdpkit.protocol._types import (
    Page,
)
from cdpkit.protocol.base import CDPEvent


class DownloadWillBegin(CDPEvent):
    """ Fired when page is about to start a download. """

    frameId: Page.FrameId
    guid: str
    url: str
    suggestedFilename: str


class DownloadProgress(CDPEvent):
    """ Fired when download makes progress. Last call has |done| == true. """

    guid: str
    totalBytes: float
    receivedBytes: float
    state: Literal['inProgress', 'completed', 'canceled']
    filePath: str | None = None  # experimental



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Runtime domain exposes JavaScript runtime by means of remote evaluation and mirror objects.
Evaluation results are returned as mirror object that expose object type, string representation
and unique identifier that can be used for further object reference. Original objects are
maintained in memory unless they are either explicitly released or are released along with the
other objects in their object group.
https://chromedevtools.github.io/devtools-protocol/tot/Runtime/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Runtime,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class AwaitPromiseInput(InputModel):

    promiseObjectId: Runtime.RemoteObjectId
    returnByValue: bool | None = None
    generatePreview: bool | None = None


class AwaitPromiseOutput(OutputModel):

    result: Runtime.RemoteObject
    exceptionDetails: Runtime.ExceptionDetails | None = None


class AwaitPromise(CDPMethod[AwaitPromiseOutput]):
    """ Add handler to promise with given promise object id. """

    INPUT_VALIDATOR = AwaitPromiseInput
    OUTPUT_VALIDATOR = AwaitPromiseOutput

    def __init__(
        self,
        *,
        promise_object_id: Runtime.RemoteObjectId,
        return_by_value: bool | None = None,
        generate_preview: bool | None = None
    ):
        super().__init__(
            promiseObjectId=promise_object_id,
            returnByValue=return_by_value,
            generatePreview=generate_preview
        )


class CallFunctionOnInput(InputModel):

    functionDeclaration: str
    objectId: Runtime.RemoteObjectId | None = None
    arguments: list[Runtime.CallArgument] | None = None
    silent: bool | None = None
    returnByValue: bool | None = None
    generatePreview: bool | None = None  # experimental
    userGesture: bool | None = None
    awaitPromise: bool | None = None
    executionContextId: Runtime.ExecutionContextId | None = None
    objectGroup: str | None = None
    throwOnSideEffect: bool | None = None  # experimental
    uniqueContextId: str | None = None  # experimental
    serializationOptions: Runtime.SerializationOptions | None = None  # experimental


class CallFunctionOnOutput(OutputModel):

    result: Runtime.RemoteObject
    exceptionDetails: Runtime.ExceptionDetails | None = None


class CallFunctionOn(CDPMethod[CallFunctionOnOutput]):
    """ Calls function with given declaration on the given object. Object group of the result is
    inherited from the target object. """

    INPUT_VALIDATOR = CallFunctionOnInput
    OUTPUT_VALIDATOR = CallFunctionOnOutput

    def __init__(
        self,
        *,
        function_declaration: str,
        object_id: Runtime.RemoteObjectId | None = None,
        arguments: list[Runtime.CallArgument] | None = None,
        silent: bool | None = None,
        return_by_value: bool | None = None,
        generate_preview: bool | None = None,
        user_gesture: bool | None = None,
        await_promise: bool | None = None,
        execution_context_id: Runtime.ExecutionContextId | None = None,
        object_group: str | None = None,
        throw_on_side_effect: bool | None = None,
        unique_context_id: str | None = None,
        serialization_options: Runtime.SerializationOptions | None = None
    ):
        super().__init__(
            functionDeclaration=function_declaration,
            objectId=object_id,
            arguments=arguments,
            silent=silent,
            returnByValue=return_by_value,
            generatePreview=generate_preview,
            userGesture=user_gesture,
            awaitPromise=await_promise,
            executionContextId=execution_context_id,
            objectGroup=object_group,
            throwOnSideEffect=throw_on_side_effect,
            uniqueContextId=unique_context_id,
            serializationOptions=serialization_options
        )


class CompileScriptInput(InputModel):

    expression: str
    sourceURL: str
    persistScript: bool
    executionContextId: Runtime.ExecutionContextId | None = None


class CompileScriptOutput(OutputModel):

    scriptId: Runtime.ScriptId | None = None
    exceptionDetails: Runtime.ExceptionDetails | None = None


class CompileScript(CDPMethod[CompileScriptOutput]):
    """ Compiles expression. """

    INPUT_VALIDATOR = CompileScriptInput
    OUTPUT_VALIDATOR = CompileScriptOutput

    def __init__(
        self,
        *,
        expression: str,
        source_url: str,
        persist_script: bool,
        execution_context_id: Runtime.ExecutionContextId | None = None
    ):
        super().__init__(
            expression=expression,
            sourceURL=source_url,
            persistScript=persist_script,
            executionContextId=execution_context_id
        )


class Disable(CDPMethod[None]):
    """ Disables reporting of execution contexts creation. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class DiscardConsoleEntries(CDPMethod[None]):
    """ Discards collected exceptions and console API calls. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables reporting of execution contexts creation by means of `executionContextCreated` event.
    When the reporting gets enabled the event will be sent immediately for each existing execution
    context. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class EvaluateInput(InputModel):

    expression: str
    objectGroup: str | None = None
    includeCommandLineAPI: bool | None = None
    silent: bool | None = None
    contextId: Runtime.ExecutionContextId | None = None
    returnByValue: bool | None = None
    generatePreview: bool | None = None  # experimental
    userGesture: bool | None = None
    awaitPromise: bool | None = None
    throwOnSideEffect: bool | None = None  # experimental
    timeout: Runtime.TimeDelta | None = None  # experimental
    disableBreaks: bool | None = None  # experimental
    replMode: bool | None = None  # experimental
    allowUnsafeEvalBlockedByCSP: bool | None = None  # experimental
    uniqueContextId: str | None = None  # experimental
    serializationOptions: Runtime.SerializationOptions | None = None  # experimental


class EvaluateOutput(OutputModel):

    result: Runtime.RemoteObject
    exceptionDetails: Runtime.ExceptionDetails | None = None


class Evaluate(CDPMethod[EvaluateOutput]):
    """ Evaluates expression on global object. """

    INPUT_VALIDATOR = EvaluateInput
    OUTPUT_VALIDATOR = EvaluateOutput

    def __init__(
        self,
        *,
        expression: str,
        object_group: str | None = None,
        include_command_line_api: bool | None = None,
        silent: bool | None = None,
        context_id: Runtime.ExecutionContextId | None = None,
        return_by_value: bool | None = None,
        generate_preview: bool | None = None,
        user_gesture: bool | None = None,
        await_promise: bool | None = None,
        throw_on_side_effect: bool | None = None,
        timeout: Runtime.TimeDelta | None = None,
        disable_breaks: bool | None = None,
        repl_mode: bool | None = None,
        allow_unsafe_eval_blocked_by_csp: bool | None = None,
        unique_context_id: str | None = None,
        serialization_options: Runtime.SerializationOptions | None = None
    ):
        super().__init__(
            expression=expression,
            objectGroup=object_group,
            includeCommandLineAPI=include_command_line_api,
            silent=silent,
            contextId=context_id,
            returnByValue=return_by_value,
            generatePreview=generate_preview,
            userGesture=user_gesture,
            awaitPromise=await_promise,
            throwOnSideEffect=throw_on_side_effect,
            timeout=timeout,
            disableBreaks=disable_breaks,
            replMode=repl_mode,
            allowUnsafeEvalBlockedByCSP=allow_unsafe_eval_blocked_by_csp,
            uniqueContextId=unique_context_id,
            serializationOptions=serialization_options
        )


class GetIsolateIdOutput(OutputModel):

    id: str


class GetIsolateId(CDPMethod[GetIsolateIdOutput]):  # experimental
    """ Returns the isolate id. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetIsolateIdOutput


class GetHeapUsageOutput(OutputModel):

    usedSize: float
    totalSize: float
    embedderHeapUsedSize: float
    backingStorageSize: float


class GetHeapUsage(CDPMethod[GetHeapUsageOutput]):  # experimental
    """ Returns the JavaScript heap usage.
    It is the total usage of the corresponding isolate not scoped to a particular Runtime. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = GetHeapUsageOutput


class GetPropertiesInput(InputModel):

    objectId: Runtime.RemoteObjectId
    ownProperties: bool | None = None
    accessorPropertiesOnly: bool | None = None  # experimental
    generatePreview: bool | None = None  # experimental
    nonIndexedPropertiesOnly: bool | None = None  # experimental


class GetPropertiesOutput(OutputModel):

    result: list[Runtime.PropertyDescriptor]
    internalProperties: list[Runtime.InternalPropertyDescriptor] | None = None
    privateProperties: list[Runtime.PrivatePropertyDescriptor] | None = None  # experimental
    exceptionDetails: Runtime.ExceptionDetails | None = None


class GetProperties(CDPMethod[GetPropertiesOutput]):
    """ Returns properties of a given object. Object group of the result is inherited from the target
    object. """

    INPUT_VALIDATOR = GetPropertiesInput
    OUTPUT_VALIDATOR = GetPropertiesOutput

    def __init__(
        self,
        *,
        object_id: Runtime.RemoteObjectId,
        own_properties: bool | None = None,
        accessor_properties_only: bool | None = None,
        generate_preview: bool | None = None,
        non_indexed_properties_only: bool | None = None
    ):
        super().__init__(
            objectId=object_id,
            ownProperties=own_properties,
            accessorPropertiesOnly=accessor_properties_only,
            generatePreview=generate_preview,
            nonIndexedPropertiesOnly=non_indexed_properties_only
        )


class GlobalLexicalScopeNamesInput(InputModel):

    executionContextId: Runtime.ExecutionContextId | None = None


class GlobalLexicalScopeNamesOutput(OutputModel):

    names: list[str]


class GlobalLexicalScopeNames(CDPMethod[GlobalLexicalScopeNamesOutput]):
    """ Returns all let, const and class variables from global scope. """

    INPUT_VALIDATOR = GlobalLexicalScopeNamesInput
    OUTPUT_VALIDATOR = GlobalLexicalScopeNamesOutput

    def __init__(
        self,
        *,
        execution_context_id: Runtime.ExecutionContextId | None = None
    ):
        super().__init__(
            executionContextId=execution_context_id
        )


class QueryObjectsInput(InputModel):

    prototypeObjectId: Runtime.RemoteObjectId
    objectGroup: str | None = None


class QueryObjectsOutput(OutputModel):

    objects: Runtime.RemoteObject


class QueryObjects(CDPMethod[QueryObjectsOutput]):

    INPUT_VALIDATOR = QueryObjectsInput
    OUTPUT_VALIDATOR = QueryObjectsOutput

    def __init__(
        self,
        *,
        prototype_object_id: Runtime.RemoteObjectId,
        object_group: str | None = None
    ):
        super().__init__(
            prototypeObjectId=prototype_object_id,
            objectGroup=object_group
        )


class ReleaseObjectInput(InputModel):

    objectId: Runtime.RemoteObjectId


class ReleaseObject(CDPMethod[None]):
    """ Releases remote object with given id. """

    INPUT_VALIDATOR = ReleaseObjectInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        object_id: Runtime.RemoteObjectId
    ):
        super().__init__(
            objectId=object_id
        )


class ReleaseObjectGroupInput(InputModel):

    objectGroup: str


class ReleaseObjectGroup(CDPMethod[None]):
    """ Releases all remote objects that belong to a given group. """

    INPUT_VALIDATOR = ReleaseObjectGroupInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        object_group: str
    ):
        super().__init__(
            objectGroup=object_group
        )


class RunIfWaitingForDebugger(CDPMethod[None]):
    """ Tells inspected instance to run if it was waiting for debugger to attach. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class RunScriptInput(InputModel):

    scriptId: Runtime.ScriptId
    executionContextId: Runtime.ExecutionContextId | None = None
    objectGroup: str | None = None
    silent: bool | None = None
    includeCommandLineAPI: bool | None = None
    returnByValue: bool | None = None
    generatePreview: bool | None = None
    awaitPromise: bool | None = None


class RunScriptOutput(OutputModel):

    result: Runtime.RemoteObject
    exceptionDetails: Runtime.ExceptionDetails | None = None


class RunScript(CDPMethod[RunScriptOutput]):
    """ Runs script with given id in a given context. """

    INPUT_VALIDATOR = RunScriptInput
    OUTPUT_VALIDATOR = RunScriptOutput

    def __init__(
        self,
        *,
        script_id: Runtime.ScriptId,
        execution_context_id: Runtime.ExecutionContextId | None = None,
        object_group: str | None = None,
        silent: bool | None = None,
        include_command_line_api: bool | None = None,
        return_by_value: bool | None = None,
        generate_preview: bool | None = None,
        await_promise: bool | None = None
    ):
        super().__init__(
            scriptId=script_id,
            executionContextId=execution_context_id,
            objectGroup=object_group,
            silent=silent,
            includeCommandLineAPI=include_command_line_api,
            returnByValue=return_by_value,
            generatePreview=generate_preview,
            awaitPromise=await_promise
        )


class SetAsyncCallStackDepthInput(InputModel):

    maxDepth: int


class SetAsyncCallStackDepth(CDPMethod[None]):
    """ Enables or disables async call stacks tracking. """

    INPUT_VALIDATOR = SetAsyncCallStackDepthInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        max_depth: int
    ):
        super().__init__(
            maxDepth=max_depth
        )


class SetCustomObjectFormatterEnabledInput(InputModel):

    enabled: bool


class SetCustomObjectFormatterEnabled(CDPMethod[None]):  # experimental

    INPUT_VALIDATOR = SetCustomObjectFormatterEnabledInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        enabled: bool
    ):
        super().__init__(
            enabled=enabled
        )


class SetMaxCallStackSizeToCaptureInput(InputModel):

    size: int


class SetMaxCallStackSizeToCapture(CDPMethod[None]):  # experimental

    INPUT_VALIDATOR = SetMaxCallStackSizeToCaptureInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        size: int
    ):
        super().__init__(
            size=size
        )


class TerminateExecution(CDPMethod[None]):  # experimental
    """ Terminate current or next JavaScript execution.
    Will cancel the termination when the outer-most script execution ends. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class AddBindingInput(InputModel):

    name: str
    executionContextId: Runtime.ExecutionContextId | None = None  # experimental deprecated
    executionContextName: str | None = None


class AddBinding(CDPMethod[None]):
    """ If executionContextId is empty, adds binding with the given name on the
    global objects of all inspected contexts, including those created later,
    bindings survive reloads.
    Binding function takes exactly one argument, this argument should be string,
    in case of any other input, function throws an exception.
    Each binding function call produces Runtime.bindingCalled notification. """

    INPUT_VALIDATOR = AddBindingInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        name: str,
        execution_context_id: Runtime.ExecutionContextId | None = None,
        execution_context_name: str | None = None
    ):
        super().__init__(
            name=name,
            executionContextId=execution_context_id,
            executionContextName=execution_context_name
        )


class RemoveBindingInput(InputModel):

    name: str


class RemoveBinding(CDPMethod[None]):
    """ This method does not remove binding function from global object but
    unsubscribes current runtime agent from Runtime.bindingCalled notifications. """

    INPUT_VALIDATOR = RemoveBindingInput
    OUTPUT_VALIDATOR = None

    def __init__(
        self,
        *,
        name: str
    ):
        super().__init__(
            name=name
        )


class GetExceptionDetailsInput(InputModel):

    errorObjectId: Runtime.RemoteObjectId


class GetExceptionDetailsOutput(OutputModel):

    exceptionDetails: Runtime.ExceptionDetails | None = None


class GetExceptionDetails(CDPMethod[GetExceptionDetailsOutput]):  # experimental
    """ This method tries to lookup and populate exception details for a
    JavaScript Error object.
    Note that the stackTrace portion of the resulting exceptionDetails will
    only be populated if the Runtime domain was enabled at the time when the
    Error was thrown. """

    INPUT_VALIDATOR = GetExceptionDetailsInput
    OUTPUT_VALIDATOR = GetExceptionDetailsOutput

    def __init__(
        self,
        *,
        error_object_id: Runtime.RemoteObjectId
    ):
        super().__init__(
            errorObjectId=error_object_id
        )



from typing import Any, TypeVar

from pydantic import BaseModel, ConfigDict

from cdpkit.logger import logger

__all__ = [
    'CDPObject',
    'InputModel',
    'OutputModel',
    'CDPEvent',
    'CDPMethod',
    'JSON_DICT',
    'RESULT_TYPE'
]

RESULT_TYPE = TypeVar('RESULT_TYPE')
JSON_DICT = dict[str, Any]


def gen_command_name(class_obj: object, remove_suffix: str) -> str:
    """Generate a DevTools protocol command name from a class object.

    Converts the class name to camelCase and combines it with the module name
    to form a command name in the format "moduleName.methodName".
    Example: Class 'ActivateTarget' in module 'network' becomes 'network.activateTarget'.

    Args:
        class_obj (str): The class object from which to generate the command name.
        remove_suffix (str): A suffix string to remove from the module name.

    Returns:
        str: The generated command name as a string in the format "module.method".
    """
    # ActivateTarget -> activateTarget
    method_name = class_obj.__name__
    method_name = f'{method_name[0].lower()}{method_name[1:]}'

    return f"{class_obj.__module__.removesuffix(remove_suffix).split('.')[-1]}.{method_name}"


class CDPObject(BaseModel):
    """
    Base class for CDP objects

    The base class for all CDP-related objects, configured in strict mode to forbid extra fields.
    """
    model_config = ConfigDict(
        extra='forbid'
    )


class InputModel(BaseModel):
    """
    Base class for input models

    The base class for input models used to validate input data, configured in strict mode to ignore extra fields.
    """
    model_config = ConfigDict(
        extra='ignore'
    )


class OutputModel(BaseModel):
    """
    Base class for output models

    The base class for output models used to validate output data, configured in strict mode to ignore extra fields.
    """
    model_config = ConfigDict(
        extra='ignore'
    )


class CDPEvent(BaseModel):
    """
    Base class for CDP events

    The base class for all CDP events, configured in strict mode to forbid extra fields.
    """
    model_config = ConfigDict(
        extra='forbid'
    )

    @classmethod
    def __pydantic_init_subclass__(cls, **kwargs):
        super().__pydantic_init_subclass__(**kwargs)
        cls.EVENT_NAME = gen_command_name(cls, remove_suffix='.events')


class CDPMethod[RESULT_TYPE]:
    """
    CDP method class

    Represents a method in the CDP protocol, containing input and output validators and command generation logic.

    Attributes:
        INPUT_VALIDATOR (InputModel | None): Input validator model
        OUTPUT_VALIDATOR (OutputModel | None): Output validator model
    """
    INPUT_VALIDATOR: InputModel | None = None
    OUTPUT_VALIDATOR: OutputModel | None = None

    def __init__(self, *_, **kwargs):
        """
        Initialize a CDP method instance

        Args:
            *_: Indicates that all positional parameters are ignored.
            **kwargs: Keyword arguments used to initialize the method's parameters.
        """
        if self.INPUT_VALIDATOR is None:
            self._params = kwargs
        else:
            input_model = self.INPUT_VALIDATOR.model_validate(kwargs)
            self._params = input_model.model_dump(exclude_none=True)
        self._command: JSON_DICT | None = None

    @property
    def command(self):
        """
        Get the command dictionary of the CDP method

        Returns:
            JSON_DICT: A dictionary containing the method name and parameters.
        """
        if self._command is None:
            self._command = {
                'method': gen_command_name(self.__class__, remove_suffix='.methods'),
                'params': self._params,
            }
        return self._command

    async def parse_response(self, response: str) -> RESULT_TYPE:
        """
        Parse the response of the CDP method

        Args:
            response (str): The response string.

        Returns:
            RESULT_TYPE: The parsed response result.
        """
        logger.info(f'Parsing response for command: {response}')
        if self.OUTPUT_VALIDATOR is None:
            return None
        else:
            return self.OUTPUT_VALIDATOR.model_validate_json(response)



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

This domain facilitates obtaining document snapshots with DOM, layout, and style information.
https://chromedevtools.github.io/devtools-protocol/tot/DOMSnapshot/

***************************************************
                    Methods
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    DOMSnapshot,
)
from cdpkit.protocol.base import CDPMethod, InputModel, OutputModel


class Disable(CDPMethod[None]):
    """ Disables DOM snapshot agent for the given page. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class Enable(CDPMethod[None]):
    """ Enables DOM snapshot agent for the given page. """

    INPUT_VALIDATOR = None
    OUTPUT_VALIDATOR = None


class GetSnapshotInput(InputModel):

    computedStyleWhitelist: list[str]
    includeEventListeners: bool | None = None
    includePaintOrder: bool | None = None
    includeUserAgentShadowTree: bool | None = None


class GetSnapshotOutput(OutputModel):

    domNodes: list[DOMSnapshot.DOMNode]
    layoutTreeNodes: list[DOMSnapshot.LayoutTreeNode]
    computedStyles: list[DOMSnapshot.ComputedStyle]


class GetSnapshot(CDPMethod[GetSnapshotOutput]):  # deprecated
    """ Returns a document snapshot, including the full DOM tree of the root node (including iframes,
    template contents, and imported documents) in a flattened array, as well as layout and
    white-listed computed style information for the nodes. Shadow DOM in the returned DOM tree is
    flattened. """

    INPUT_VALIDATOR = GetSnapshotInput
    OUTPUT_VALIDATOR = GetSnapshotOutput

    def __init__(
        self,
        *,
        computed_style_whitelist: list[str],
        include_event_listeners: bool | None = None,
        include_paint_order: bool | None = None,
        include_user_agent_shadow_tree: bool | None = None
    ):
        super().__init__(
            computedStyleWhitelist=computed_style_whitelist,
            includeEventListeners=include_event_listeners,
            includePaintOrder=include_paint_order,
            includeUserAgentShadowTree=include_user_agent_shadow_tree
        )


class CaptureSnapshotInput(InputModel):

    computedStyles: list[str]
    includePaintOrder: bool | None = None
    includeDOMRects: bool | None = None
    includeBlendedBackgroundColors: bool | None = None  # experimental
    includeTextColorOpacities: bool | None = None  # experimental


class CaptureSnapshotOutput(OutputModel):

    documents: list[DOMSnapshot.DocumentSnapshot]
    strings: list[str]


class CaptureSnapshot(CDPMethod[CaptureSnapshotOutput]):
    """ Returns a document snapshot, including the full DOM tree of the root node (including iframes,
    template contents, and imported documents) in a flattened array, as well as layout and
    white-listed computed style information for the nodes. Shadow DOM in the returned DOM tree is
    flattened. """

    INPUT_VALIDATOR = CaptureSnapshotInput
    OUTPUT_VALIDATOR = CaptureSnapshotOutput

    def __init__(
        self,
        *,
        computed_styles: list[str],
        include_paint_order: bool | None = None,
        include_dom_rects: bool | None = None,
        include_blended_background_colors: bool | None = None,
        include_text_color_opacities: bool | None = None
    ):
        super().__init__(
            computedStyles=computed_styles,
            includePaintOrder=include_paint_order,
            includeDOMRects=include_dom_rects,
            includeBlendedBackgroundColors=include_blended_background_colors,
            includeTextColorOpacities=include_text_color_opacities
        )



from .methods import CaptureSnapshot, Disable, Enable, GetSnapshot
from .types import (
    ArrayOfStrings,
    ComputedStyle,
    DocumentSnapshot,
    DOMNode,
    InlineTextBox,
    LayoutTreeNode,
    LayoutTreeSnapshot,
    NameValue,
    NodeTreeSnapshot,
    RareBooleanData,
    RareIntegerData,
    RareStringData,
    Rectangle,
    StringIndex,
    TextBoxSnapshot,
)

__all__ = [
    DOMNode,
    InlineTextBox,
    LayoutTreeNode,
    ComputedStyle,
    NameValue,
    StringIndex,
    ArrayOfStrings,
    RareStringData,
    RareBooleanData,
    RareIntegerData,
    Rectangle,
    DocumentSnapshot,
    NodeTreeSnapshot,
    LayoutTreeSnapshot,
    TextBoxSnapshot,
    Disable,
    Enable,
    GetSnapshot,
    CaptureSnapshot,
]



"""
This file is generated by generator/run.py based on the Chrome DevTools Protocol.

Audits domain allows investigation of page violations and possible improvements.
https://chromedevtools.github.io/devtools-protocol/tot/Audits/

***************************************************
                    Events
***************************************************
"""
from __future__ import annotations

from cdpkit.protocol._types import (
    Audits,
)
from cdpkit.protocol.base import CDPEvent


class IssueAdded(CDPEvent):

    issue: Audits.InspectorIssue
